# -*- coding: utf-8 -*-
'''
Support for Tomcat

This module uses the manager webapp to manage Apache tomcat webapps.
If the manager webapp is not configured some of the functions won't work.

:configuration:
    - Java bin path should be in default path
    - If ipv6 is enabled make sure you permit manager access to ipv6 interface
      "0:0:0:0:0:0:0:1"
    - If you are using tomcat.tar.gz it has to be installed or symlinked under
      ``/opt``, preferably using name tomcat
    - "tomcat.signal start/stop" works but it does not use the startup scripts

The following grains/pillar should be set:

.. code-block:: yaml

    tomcat-manager:
      user: <username>
      passwd: <password>

or the old format:

.. code-block:: yaml

    tomcat-manager.user: <username>
    tomcat-manager.passwd: <password>

Also configure a user in the conf/tomcat-users.xml file:

.. code-block:: xml

    <?xml version='1.0' encoding='utf-8'?>
    <tomcat-users>
        <role rolename="manager-script"/>
        <user username="tomcat" password="tomcat" roles="manager-script"/>
    </tomcat-users>

.. note::

   - More information about tomcat manager:
     http://tomcat.apache.org/tomcat-7.0-doc/manager-howto.html
   - if you use only this module for deployments you've might want to strict
     access to the manager only from localhost for more info:
     http://tomcat.apache.org/tomcat-7.0-doc/manager-howto.html#Configuring_Manager_Application_Access
   - Tested on:

     JVM Vendor:
         Sun Microsystems Inc.
     JVM Version:
         1.6.0_43-b01
     OS Architecture:
         amd64
     OS Name:
         Linux
     OS Version:
         2.6.32-358.el6.x86_64
     Tomcat Version:
         Apache Tomcat/7.0.37
'''
from __future__ import absolute_import

# Import python libs
import os
import re
import glob
import hashlib
import tempfile
import logging

# Import 3rd-party libs
# pylint: disable=no-name-in-module,import-error
from salt.ext.six.moves.urllib.parse import urlencode as _urlencode
from salt.ext.six.moves.urllib.request import (
        urlopen as _urlopen,
        HTTPBasicAuthHandler as _HTTPBasicAuthHandler,
        HTTPDigestAuthHandler as _HTTPDigestAuthHandler,
        build_opener as _build_opener,
        install_opener as _install_opener
)
# pylint: enable=no-name-in-module,import-error

# Import Salt libs
import salt.utils

log = logging.getLogger(__name__)

__func_alias__ = {
    'reload_': 'reload'
}

# Support old-style grains/pillar
# config as well as new.
__valid_configs = {
    'user': [
        'tomcat-manager.user',
        'tomcat-manager:user'
    ],
    'passwd': [
        'tomcat-manager.passwd',
        'tomcat-manager:passwd'
    ]
}


def __virtual__():
    '''
    Only load tomcat if it is installed or if grains/pillar config exists
    '''
    if __catalina_home() or _auth('dummy'):
        return 'tomcat'
    return False


def __catalina_home():
    '''
    Tomcat paths differ depending on packaging
    '''
    locations = ['/usr/share/tomcat*', '/opt/tomcat']
    for location in locations:
        folders = glob.glob(location)
        if folders:
            for catalina_home in folders:
                if os.path.isdir(catalina_home + "/bin"):
                    return catalina_home
    return False


def _get_credentials():
    '''
    Get the username and password from opts, grains, or pillar
    '''
    ret = {
        'user': False,
        'passwd': False
    }

    # Loop through opts, grains, and pillar
    # Return the first acceptable configuration found
    for item in ret:
        for struct in [__opts__, __grains__, __pillar__]:
            # Look for the config key
            # Support old-style config format and new
            for config_key in __valid_configs[item]:
                value = salt.utils.traverse_dict_and_list(struct,
                                                          config_key,
                                                          None)
                if value:
                    ret[item] = value
                    break
    return ret['user'], ret['passwd']


def _auth(uri):
    '''
    returns a authentication handler.
    Get user & password from grains, if are not set default to
    modules.config.option

    If user & pass are missing return False
    '''

    user, password = _get_credentials()
    if user is False or password is False:
        return False

    basic = _HTTPBasicAuthHandler()
    basic.add_password(realm='Tomcat Manager Application', uri=uri,
            user=user, passwd=password)
    digest = _HTTPDigestAuthHandler()
    digest.add_password(realm='Tomcat Manager Application', uri=uri,
            user=user, passwd=password)
    return _build_opener(basic, digest)


def _extract_war_version(war):
    '''
    Extract the version from the war file name.  There does not seem to be a
    standard for encoding the version into the `war file name
    <https://tomcat.apache.org/tomcat-6.0-doc/deployer-howto.html>`_.

    Examples:

    .. code-block::

        /path/salt-2015.8.6.war -> 2015.8.6
        /path/V6R2013xD5.war -> V6R2013xD5
    '''
    basename = os.path.basename(war)
    war_package = os.path.splitext(basename)[0]  # remove '.war'
    version = re.findall("-([\\d.-]+)$", war_package)  # try semver
    return version[0] if version and len(version) == 1 else war_package  # default to whole name


def _wget(cmd, opts=None, url='http://localhost:8080/manager', timeout=180):
    '''
    A private function used to issue the command to tomcat via the manager
    webapp

    cmd
        the command to execute
    url
        the URL of the server manager webapp
        example: http://localhost:8080/manager
    opts
        a dict of arguments
    timeout
        timeout for HTTP request

    return value is a dict in the from of::

        {
            res: [True|False]
            msg: list of lines we got back from the manager
        }
    '''

    ret = {
        'res': True,
        'msg': []
    }

    # prepare authentication
    auth = _auth(url)
    if auth is False:
        ret['res'] = False
        ret['msg'] = 'missing username and password settings (grain/pillar)'
        return ret

    # prepare URL
    if url[-1] != '/':
        url += '/'
    url6 = url
    url += 'text/{0}'.format(cmd)
    url6 += '{0}'.format(cmd)
    if opts:
        url += '?{0}'.format(_urlencode(opts))
        url6 += '?{0}'.format(_urlencode(opts))

    # Make the HTTP request
    _install_opener(auth)

    try:
        # Trying tomcat >= 7 url
        ret['msg'] = _urlopen(url, timeout=timeout).read().splitlines()
    except Exception:
        try:
            # Trying tomcat6 url
            ret['msg'] = _urlopen(url6, timeout=timeout).read().splitlines()
        except Exception:
            ret['msg'] = 'Failed to create HTTP request'

    if not ret['msg'][0].startswith('OK'):
        ret['res'] = False

    return ret


def _simple_cmd(cmd, app, url='http://localhost:8080/manager', timeout=180):
    '''
    Simple command wrapper to commands that need only a path option
    '''

    try:
        opts = {
            'path': app,
            'version': ls(url)[app]['version']
        }
        return '\n'.join(_wget(cmd, opts, url, timeout=timeout)['msg'])
    except Exception:
        return 'FAIL - No context exists for path {0}'.format(app)


# Functions
def leaks(url='http://localhost:8080/manager', timeout=180):
    '''
    Find memory leaks in tomcat

    url : http://localhost:8080/manager
        the URL of the server manager webapp
    timeout : 180
        timeout for HTTP request

    CLI Examples:

    .. code-block:: bash

        salt '*' tomcat.leaks
    '''

    return _wget('findleaks', {'statusLine': 'true'},
        url, timeout=timeout)['msg']


def status(url='http://localhost:8080/manager', timeout=180):
    '''
    Used to test if the tomcat manager is up

    url : http://localhost:8080/manager
        the URL of the server manager webapp
    timeout : 180
        timeout for HTTP request

    CLI Examples:

    .. code-block:: bash

        salt '*' tomcat.status
        salt '*' tomcat.status http://localhost:8080/manager
    '''

    return _wget('list', {}, url, timeout=timeout)['res']


def ls(url='http://localhost:8080/manager', timeout=180):
    '''
    list all the deployed webapps

    url : http://localhost:8080/manager
        the URL of the server manager webapp
    timeout : 180
        timeout for HTTP request

    CLI Examples:

    .. code-block:: bash

        salt '*' tomcat.ls
        salt '*' tomcat.ls http://localhost:8080/manager
    '''

    ret = {}
    data = _wget('list', '', url, timeout=timeout)
    if data['res'] is False:
        return {}
    data['msg'].pop(0)
    for line in data['msg']:
        tmp = line.split(':')
        ret[tmp[0]] = {
            'mode': tmp[1],
            'sessions': tmp[2],
            'fullname': tmp[3],
            'version': '',
        }
        sliced = tmp[3].split('##')
        if len(sliced) > 1:
            ret[tmp[0]]['version'] = sliced[1]

    return ret


def stop(app, url='http://localhost:8080/manager', timeout=180):
    '''
    Stop the webapp

    app
        the webapp context path
    url : http://localhost:8080/manager
        the URL of the server manager webapp
    timeout : 180
        timeout for HTTP request

    CLI Examples:

    .. code-block:: bash

        salt '*' tomcat.stop /jenkins
        salt '*' tomcat.stop /jenkins http://localhost:8080/manager
    '''

    return _simple_cmd('stop', app, url, timeout=timeout)


def start(app, url='http://localhost:8080/manager', timeout=180):
    '''
    Start the webapp

    app
        the webapp context path
    url : http://localhost:8080/manager
        the URL of the server manager webapp
    timeout
        timeout for HTTP request

    CLI Examples:

    .. code-block:: bash

        salt '*' tomcat.start /jenkins
        salt '*' tomcat.start /jenkins http://localhost:8080/manager
    '''

    return _simple_cmd('start', app, url, timeout=timeout)


def reload_(app, url='http://localhost:8080/manager', timeout=180):
    '''
    Reload the webapp

    app
        the webapp context path
    url : http://localhost:8080/manager
        the URL of the server manager webapp
    timeout : 180
        timeout for HTTP request

    CLI Examples:

    .. code-block:: bash

        salt '*' tomcat.reload /jenkins
        salt '*' tomcat.reload /jenkins http://localhost:8080/manager
    '''

    return _simple_cmd('reload', app, url, timeout=timeout)


def sessions(app, url='http://localhost:8080/manager', timeout=180):
    '''
    return the status of the webapp sessions

    app
        the webapp context path
    url : http://localhost:8080/manager
        the URL of the server manager webapp
    timeout : 180
        timeout for HTTP request

    CLI Examples:

    .. code-block:: bash

        salt '*' tomcat.sessions /jenkins
        salt '*' tomcat.sessions /jenkins http://localhost:8080/manager
    '''

    return _simple_cmd('sessions', app, url, timeout=timeout)


def status_webapp(app, url='http://localhost:8080/manager', timeout=180):
    '''
    return the status of the webapp (stopped | running | missing)

    app
        the webapp context path
    url : http://localhost:8080/manager
        the URL of the server manager webapp
    timeout : 180
        timeout for HTTP request

    CLI Examples:

    .. code-block:: bash

        salt '*' tomcat.status_webapp /jenkins
        salt '*' tomcat.status_webapp /jenkins http://localhost:8080/manager
    '''

    webapps = ls(url, timeout=timeout)
    for i in webapps:
        if i == app:
            return webapps[i]['mode']

    return 'missing'


def serverinfo(url='http://localhost:8080/manager', timeout=180):
    '''
    return details about the server

    url : http://localhost:8080/manager
        the URL of the server manager webapp
    timeout : 180
        timeout for HTTP request

    CLI Examples:

    .. code-block:: bash

        salt '*' tomcat.serverinfo
        salt '*' tomcat.serverinfo http://localhost:8080/manager
    '''

    data = _wget('serverinfo', {}, url, timeout=timeout)
    if data['res'] is False:
        return {'error': data['msg']}

    ret = {}
    data['msg'].pop(0)
    for line in data['msg']:
        tmp = line.split(':')
        ret[tmp[0].strip()] = tmp[1].strip()

    return ret


def undeploy(app, url='http://localhost:8080/manager', timeout=180):
    '''
    Undeploy a webapp

    app
        the webapp context path
    url : http://localhost:8080/manager
        the URL of the server manager webapp
    timeout : 180
        timeout for HTTP request

    CLI Examples:

    .. code-block:: bash

        salt '*' tomcat.undeploy /jenkins
        salt '*' tomcat.undeploy /jenkins http://localhost:8080/manager
    '''

    return _simple_cmd('undeploy', app, url, timeout=timeout)


def deploy_war(war,
               context,
               force='no',
               url='http://localhost:8080/manager',
               saltenv='base',
               timeout=180,
               env=None,
               temp_war_location=None,
               version=''):
    '''
    Deploy a WAR file

    war
        absolute path to WAR file (should be accessible by the user running
        tomcat) or a path supported by the salt.modules.cp.get_file function
    context
        the context path to deploy
    force : False
        set True to deploy the webapp even one is deployed in the context
    url : http://localhost:8080/manager
        the URL of the server manager webapp
    saltenv : base
        the environment for WAR file in used by salt.modules.cp.get_url
        function
    timeout : 180
        timeout for HTTP request
    temp_war_location : None
        use another location to temporarily copy to war file
        by default the system's temp directory is used
    version : ''
        Specify the war version.  If this argument is provided, it overrides
        the version encoded in the war file name, if one is present.

        Examples:

        .. code-block:: bash

            salt '*' tomcat.deploy_war salt://salt-2015.8.6.war version=2015.08.r6

        .. versionadded:: 2015.8.6

    CLI Examples:

    cp module

    .. code-block:: bash

        salt '*' tomcat.deploy_war salt://application.war /api
        salt '*' tomcat.deploy_war salt://application.war /api no
        salt '*' tomcat.deploy_war salt://application.war /api yes http://localhost:8080/manager

    minion local file system

    .. code-block:: bash

        salt '*' tomcat.deploy_war /tmp/application.war /api
        salt '*' tomcat.deploy_war /tmp/application.war /api no
        salt '*' tomcat.deploy_war /tmp/application.war /api yes http://localhost:8080/manager
    '''
    if env is not None:
        salt.utils.warn_until(
            'Boron',
            'Passing a salt environment should be done using \'saltenv\' '
            'not \'env\'. This functionality will be removed in Salt Boron.'
        )
        # Backwards compatibility
        saltenv = env

    # Decide the location to copy the war for the deployment
    tfile = 'salt.{0}'.format(os.path.basename(war))
    if temp_war_location is not None:
        if not os.path.isdir(temp_war_location):
            return 'Error - "{0}" is not a directory'.format(temp_war_location)
        tfile = os.path.join(temp_war_location, tfile)
    else:
        tfile = os.path.join(tempfile.gettempdir(), tfile)

    # Copy file name if needed
    cache = False
    if not os.path.isfile(war):
        cache = True
        cached = __salt__['cp.get_url'](war, tfile, saltenv)
        if not cached:
            return 'FAIL - could not cache the WAR file'
        try:
            __salt__['file.set_mode'](cached, '0644')
        except KeyError:
            pass
    else:
        tfile = war

    # Prepare options
    opts = {
        'war': 'file:{0}'.format(tfile),
        'path': context,
        'version': version or _extract_war_version(war),
    }
    if force == 'yes':
        opts['update'] = 'true'

    # Deploy
    deployed = _wget('deploy', opts, url, timeout=timeout)
    res = '\n'.join(deployed['msg'])

    # Cleanup
    if cache:
        __salt__['file.remove'](tfile)

    return res


def passwd(passwd,
           user='',
           alg='sha1',
           realm=None):
    '''
    This function replaces the $CATALINA_HOME/bin/digest.sh script
    convert a clear-text password to the $CATALINA_BASE/conf/tomcat-users.xml
    format

    CLI Examples:

    .. code-block:: bash

        salt '*' tomcat.passwd secret
        salt '*' tomcat.passwd secret tomcat sha1
        salt '*' tomcat.passwd secret tomcat sha1 'Protected Realm'
    '''
    # Shouldn't it be SHA265 instead of SHA1?
    digest = hasattr(hashlib, alg) and getattr(hashlib, alg) or None
    if digest:
        if realm:
            digest.update('{0}:{1}:{2}'.format(user, realm, passwd,))
        else:
            digest.update(passwd)

    return digest and digest.hexdigest() or False


# Non-Manager functions
def version():
    '''
    Return server version from catalina.sh version

    CLI Example:

    .. code-block:: bash

        salt '*' tomcat.version
    '''
    cmd = __catalina_home() + '/bin/catalina.sh version'
    out = __salt__['cmd.run'](cmd).splitlines()
    for line in out:
        if not line:
            continue
        if 'Server version' in line:
            comps = line.split(': ')
            return comps[1]


def fullversion():
    '''
    Return all server information from catalina.sh version

    CLI Example:

    .. code-block:: bash

        salt '*' tomcat.fullversion
    '''
    cmd = __catalina_home() + '/bin/catalina.sh version'
    ret = {}
    out = __salt__['cmd.run'](cmd).splitlines()
    for line in out:
        if not line:
            continue
        if ': ' in line:
            comps = line.split(': ')
            ret[comps[0]] = comps[1].lstrip()
    return ret


def signal(signal=None):
    '''
    Signals catalina to start, stop, securestart, forcestop.

    CLI Example:

    .. code-block:: bash

        salt '*' tomcat.signal start
    '''
    valid_signals = {'forcestop': 'stop -force',
                     'securestart': 'start -security',
                     'start': 'start',
                     'stop': 'stop'}

    if signal not in valid_signals:
        return

    cmd = '{0}/bin/catalina.sh {1}'.format(
        __catalina_home(), valid_signals[signal]
    )
    __salt__['cmd.run'](cmd)


if __name__ == '__main__':
    '''
    Allow testing from the CLI
    '''  # pylint: disable=W0105
    __opts__ = {}
    __grains__ = {}
    __pillar__ = {
        'tomcat-manager.user': 'foobar',
        'tomcat-manager.passwd': 'barfoo1!',
    }

    old_format_creds = _get_credentials()

    __pillar__ = {
        'tomcat-manager': {
            'user': 'foobar',
            'passwd': 'barfoo1!'
        }
    }

    new_format_creds = _get_credentials()

    if old_format_creds == new_format_creds:
        log.info('Config backwards compatible')
    else:
        log.ifno('Config not backwards compatible')



# -*- coding: utf-8 -*-
'''
Utility functions for the rest_sample
'''
from __future__ import absolute_import

__proxyenabled__ = ['rest_sample']


def fix_outage():
    '''
    "Fix" the outage

    CLI Example:

    .. code-block:: bash

        salt 'rest-sample-proxy' rest_sample.fix_outage

    '''
    return __proxy__['rest_sample.fix_outage']()



# -*- coding: utf-8 -*-
'''
    :codeauthor: :email:`Jayesh Kariya <jayeshk@saltstack.com>`
'''
# Import Python libs
from __future__ import absolute_import

# Import Salt Testing Libs
from salttesting import skipIf, TestCase
from salttesting.mock import (
    NO_MOCK,
    NO_MOCK_REASON,
    MagicMock,
    patch
)

from salttesting.helpers import ensure_in_syspath

ensure_in_syspath('../../')

# Import Salt Libs
from salt.states import status

status.__salt__ = {}


@skipIf(NO_MOCK, NO_MOCK_REASON)
class StatusTestCase(TestCase):
    '''
    Test cases for salt.states.status
    '''
    # 'loadavg' function tests: 1

    def test_loadavg(self):
        '''
        Test to return the current load average for the specified minion.
        '''
        name = 'mymonitor'

        ret = {'name': name,
               'changes': {},
               'result': True,
               'data': {},
               'comment': ''}

        mock = MagicMock(return_value=[])
        with patch.dict(status.__salt__, {'status.loadavg': mock}):
            comt = ('Requested load average mymonitor not available ')
            ret.update({'comment': comt, 'result': False})
            self.assertDictEqual(status.loadavg(name), ret)

        mock = MagicMock(return_value={name: 3})
        with patch.dict(status.__salt__, {'status.loadavg': mock}):
            comt = ('Min must be less than max')
            ret.update({'comment': comt, 'result': False})
            self.assertDictEqual(status.loadavg(name, 1, 5), ret)

            comt = ('Load avg is below minimum of 4 at 3.0')
            ret.update({'comment': comt, 'data': 3})
            self.assertDictEqual(status.loadavg(name, 5, 4), ret)

            comt = ('Load avg above maximum of 2 at 3.0')
            ret.update({'comment': comt, 'data': 3})
            self.assertDictEqual(status.loadavg(name, 2, 1), ret)

            comt = ('Load avg in acceptable range')
            ret.update({'comment': comt, 'result': True})
            self.assertDictEqual(status.loadavg(name, 3, 1), ret)

    # 'process' function tests: 1

    def test_process(self):
        '''
        Test to return whether the specified signature
        is found in the process tree.
        '''
        name = 'mymonitor'

        ret = {'name': name,
               'changes': {},
               'result': True,
               'data': {},
               'comment': ''}

        mock = MagicMock(side_effect=[{}, {name: 1}])
        with patch.dict(status.__salt__, {'status.pid': mock}):
            comt = ('Process signature "mymonitor" not found ')
            ret.update({'comment': comt, 'result': False})
            self.assertDictEqual(status.process(name), ret)

            comt = ('Process signature "mymonitor" was found ')
            ret.update({'comment': comt, 'result': True,
                        'data': {name: 1}})
            self.assertDictEqual(status.process(name), ret)


if __name__ == '__main__':
    from integration import run_tests
    run_tests(StatusTestCase, needs_daemon=False)



# -*- coding: utf-8 -*-
'''
Return config information
'''

# Import python libs
from __future__ import absolute_import
import copy
import re
import os
import logging

# Import salt libs
import salt.config
import salt.utils
try:
    # Gated for salt-ssh (salt.utils.cloud imports msgpack)
    import salt.utils.cloud
    HAS_CLOUD = True
except ImportError:
    HAS_CLOUD = False

import salt._compat
import salt.syspaths as syspaths
import salt.utils.sdb as sdb

# Import 3rd-party libs
import salt.ext.six as six

log = logging.getLogger(__name__)

__proxyenabled__ = ['*']

# Set up the default values for all systems
DEFAULTS = {'mongo.db': 'salt',
            'mongo.host': 'salt',
            'mongo.password': '',
            'mongo.port': 27017,
            'mongo.user': '',
            'redis.db': '0',
            'redis.host': 'salt',
            'redis.port': 6379,
            'test.foo': 'unconfigured',
            'ca.cert_base_path': '/etc/pki',
            'solr.cores': [],
            'solr.host': 'localhost',
            'solr.port': '8983',
            'solr.baseurl': '/solr',
            'solr.type': 'master',
            'solr.request_timeout': None,
            'solr.init_script': '/etc/rc.d/solr',
            'solr.dih.import_options': {'clean': False, 'optimize': True,
                                        'commit': True, 'verbose': False},
            'solr.backup_path': None,
            'solr.num_backups': 1,
            'poudriere.config': '/usr/local/etc/poudriere.conf',
            'poudriere.config_dir': '/usr/local/etc/poudriere.d',
            'ldap.uri': '',
            'ldap.server': 'localhost',
            'ldap.port': '389',
            'ldap.tls': False,
            'ldap.no_verify': False,
            'ldap.anonymous': True,
            'ldap.scope': 2,
            'ldap.attrs': None,
            'ldap.binddn': '',
            'ldap.bindpw': '',
            'hosts.file': '/etc/hosts',
            'aliases.file': '/etc/aliases',
            'virt.images': os.path.join(syspaths.SRV_ROOT_DIR, 'salt-images'),
            'virt.tunnel': False,
            }


def backup_mode(backup=''):
    '''
    Return the backup mode

    CLI Example:

    .. code-block:: bash

        salt '*' config.backup_mode
    '''
    if backup:
        return backup
    return option('backup_mode')


def manage_mode(mode):
    '''
    Return a mode value, normalized to a string

    CLI Example:

    .. code-block:: bash

        salt '*' config.manage_mode
    '''
    if mode is None:
        return None
    if not isinstance(mode, six.string_types):
        # Make it a string in case it's not
        mode = str(mode)
    # Strip any quotes and initial 0, though zero-pad it up to 4
    ret = mode.strip('"').strip('\'').lstrip('0').zfill(4)
    if ret[0] != '0':
        # Always include a leading zero
        return '0{0}'.format(ret)
    return ret


def valid_fileproto(uri):
    '''
    Returns a boolean value based on whether or not the URI passed has a valid
    remote file protocol designation

    CLI Example:

    .. code-block:: bash

        salt '*' config.valid_fileproto salt://path/to/file
    '''
    try:
        return bool(re.match('^(?:salt|https?|ftp)://', uri))
    except Exception:
        return False


def option(
        value,
        default='',
        omit_opts=False,
        omit_master=False,
        omit_pillar=False):
    '''
    Pass in a generic option and receive the value that will be assigned

    CLI Example:

    .. code-block:: bash

        salt '*' config.option redis.host
    '''
    if not omit_opts:
        if value in __opts__:
            return __opts__[value]
    if not omit_master:
        if value in __pillar__.get('master', {}):
            return __pillar__['master'][value]
    if not omit_pillar:
        if value in __pillar__:
            return __pillar__[value]
    if value in DEFAULTS:
        return DEFAULTS[value]
    return default


def merge(value,
          default='',
          omit_opts=False,
          omit_master=False,
          omit_pillar=False):
    '''
    Retrieves an option based on key, merging all matches.

    Same as ``option()`` except that it merges all matches, rather than taking
    the first match.

    CLI Example:

    .. code-block:: bash

        salt '*' config.merge schedule
    '''
    ret = None
    if not omit_opts:
        if value in __opts__:
            ret = __opts__[value]
            if isinstance(ret, str):
                return ret
    if not omit_master:
        if value in __pillar__.get('master', {}):
            tmp = __pillar__['master'][value]
            if ret is None:
                ret = tmp
                if isinstance(ret, str):
                    return ret
            elif isinstance(ret, dict) and isinstance(tmp, dict):
                tmp.update(ret)
                ret = tmp
            elif isinstance(ret, (list, tuple)) and isinstance(tmp,
                                                               (list, tuple)):
                ret = list(ret) + list(tmp)
    if not omit_pillar:
        if value in __pillar__:
            tmp = __pillar__[value]
            if ret is None:
                ret = tmp
                if isinstance(ret, str):
                    return ret
            elif isinstance(ret, dict) and isinstance(tmp, dict):
                tmp.update(ret)
                ret = tmp
            elif isinstance(ret, (list, tuple)) and isinstance(tmp,
                                                               (list, tuple)):
                ret = list(ret) + list(tmp)
    if ret is None and value in DEFAULTS:
        return DEFAULTS[value]
    return ret or default


def get(key, default='', delimiter=':', merge=None):
    '''
    .. versionadded: 0.14.0

    Attempt to retrieve the named value from the minion config file, pillar,
    grains or the master config. If the named value is not available, return the
    value specified by ``default``. If not specified, the default is an empty
    string.

    Values can also be retrieved from nested dictionaries. Assume the below
    data structure:

    .. code-block:: python

        {'pkg': {'apache': 'httpd'}}

    To retrieve the value associated with the ``apache`` key, in the
    sub-dictionary corresponding to the ``pkg`` key, the following command can
    be used:

    .. code-block:: bash

        salt myminion config.get pkg:apache

    The ``:`` (colon) is used to represent a nested dictionary level.

    .. versionchanged:: 2015.5.0
        The ``delimiter`` argument was added, to allow delimiters other than
        ``:`` to be used.

    This function traverses these data stores in this order, returning the
    first match found:

    - Minion configuration
    - Minion's grains
    - Minion's pillar data
    - Master configuration (requires :conf_minion:`pillar_opts` to be set to
      ``True`` in Minion config file in order to work)

    This means that if there is a value that is going to be the same for the
    majority of minions, it can be configured in the Master config file, and
    then overridden using the grains, pillar, or Minion config file.

    Adding config options to the Master or Minion configuration file is easy:

    .. code-block:: yaml

        my-config-option: value
        cafe-menu:
          - egg and bacon
          - egg sausage and bacon
          - egg and spam
          - egg bacon and spam
          - egg bacon sausage and spam
          - spam bacon sausage and spam
          - spam egg spam spam bacon and spam
          - spam sausage spam spam bacon spam tomato and spam

    .. note::
        Minion configuration options built into Salt (like those defined
        :ref:`here <configuration-salt-minion>`) will *always* be defined in
        the Minion configuration and thus *cannot be overridden by grains or
        pillar data*. However, additional (user-defined) configuration options
        (as in the above example) will not be in the Minion configuration by
        default and thus can be overridden using grains/pillar data by leaving
        the option out of the minion config file.

    **Arguments**

    delimiter
        .. versionadded:: 2015.5.0

        Override the delimiter used to separate nested levels of a data
        structure.

    merge
        .. versionadded:: 2015.5.0

        If passed, this parameter will change the behavior of the function so
        that, instead of traversing each data store above in order and
        returning the first match, the data stores are first merged together
        and then searched. The pillar data is merged into the master config
        data, then the grains are merged, followed by the Minion config data.
        The resulting data structure is then searched for a match. This allows
        for configurations to be more flexible.

        .. note::

            The merging described above does not mean that grain data will end
            up in the Minion's pillar data, or pillar data will end up in the
            master config data, etc. The data is just combined for the purposes
            of searching an amalgam of the different data stores.

        The supported merge strategies are as follows:

        - **recurse** - If a key exists in both dictionaries, and the new value
          is not a dictionary, it is replaced. Otherwise, the sub-dictionaries
          are merged together into a single dictionary, recursively on down,
          following the same criteria. For example:

          .. code-block:: python

              >>> dict1 = {'foo': {'bar': 1, 'qux': True},
                           'hosts': ['a', 'b', 'c'],
                           'only_x': None}
              >>> dict2 = {'foo': {'baz': 2, 'qux': False},
                           'hosts': ['d', 'e', 'f'],
                           'only_y': None}
              >>> merged
              {'foo': {'bar': 1, 'baz': 2, 'qux': False},
               'hosts': ['d', 'e', 'f'],
               'only_dict1': None,
               'only_dict2': None}

        - **overwrite** - If a key exists in the top level of both
          dictionaries, the new value completely overwrites the old. For
          example:

          .. code-block:: python

              >>> dict1 = {'foo': {'bar': 1, 'qux': True},
                           'hosts': ['a', 'b', 'c'],
                           'only_x': None}
              >>> dict2 = {'foo': {'baz': 2, 'qux': False},
                           'hosts': ['d', 'e', 'f'],
                           'only_y': None}
              >>> merged
              {'foo': {'baz': 2, 'qux': False},
               'hosts': ['d', 'e', 'f'],
               'only_dict1': None,
               'only_dict2': None}

    CLI Example:

    .. code-block:: bash

        salt '*' config.get pkg:apache
        salt '*' config.get lxc.container_profile:centos merge=recurse
    '''
    if merge is None:
        ret = salt.utils.traverse_dict_and_list(__opts__,
                                                key,
                                                '_|-',
                                                delimiter=delimiter)
        if ret != '_|-':
            return sdb.sdb_get(ret, __opts__)

        ret = salt.utils.traverse_dict_and_list(__grains__,
                                                key,
                                                '_|-',
                                                delimiter)
        if ret != '_|-':
            return sdb.sdb_get(ret, __opts__)

        ret = salt.utils.traverse_dict_and_list(__pillar__,
                                                key,
                                                '_|-',
                                                delimiter=delimiter)
        if ret != '_|-':
            return sdb.sdb_get(ret, __opts__)

        ret = salt.utils.traverse_dict_and_list(__pillar__.get('master', {}),
                                                key,
                                                '_|-',
                                                delimiter=delimiter)
        if ret != '_|-':
            return sdb.sdb_get(ret, __opts__)
    else:
        if merge not in ('recurse', 'overwrite'):
            log.warning('Unsupported merge strategy \'{0}\'. Falling back '
                        'to \'recurse\'.'.format(merge))
            merge = 'recurse'

        merge_lists = salt.config.master_config('/etc/salt/master').get('pillar_merge_lists')

        data = copy.copy(__pillar__.get('master', {}))
        data = salt.utils.dictupdate.merge(data, __pillar__, strategy=merge, merge_lists=merge_lists)
        data = salt.utils.dictupdate.merge(data, __grains__, strategy=merge, merge_lists=merge_lists)
        data = salt.utils.dictupdate.merge(data, __opts__, strategy=merge, merge_lists=merge_lists)
        ret = salt.utils.traverse_dict_and_list(data,
                                                key,
                                                '_|-',
                                                delimiter=delimiter)
        if ret != '_|-':
            return sdb.sdb_get(ret, __opts__)

    return default


def dot_vals(value):
    '''
    Pass in a configuration value that should be preceded by the module name
    and a dot, this will return a list of all read key/value pairs

    CLI Example:

    .. code-block:: bash

        salt '*' config.dot_vals host
    '''
    ret = {}
    for key, val in six.iteritems(__pillar__.get('master', {})):
        if key.startswith('{0}.'.format(value)):
            ret[key] = val
    for key, val in six.iteritems(__opts__):
        if key.startswith('{0}.'.format(value)):
            ret[key] = val
    return ret


def gather_bootstrap_script(bootstrap=None):
    '''
    Download the salt-bootstrap script, and return its location

    bootstrap
        URL of alternate bootstrap script

    CLI Example:

    .. code-block:: bash

        salt '*' config.gather_bootstrap_script
    '''
    if not HAS_CLOUD:
        return False, 'config.gather_bootstrap_script is unavailable'
    ret = salt.utils.cloud.update_bootstrap(__opts__, url=bootstrap)
    if 'Success' in ret and len(ret['Success']['Files updated']) > 0:
        return ret['Success']['Files updated'][0]



# -*- coding: utf-8 -*-
'''
Managing python installations with pyenv
========================================

This module is used to install and manage python installations with pyenv.
Different versions of python can be installed, and uninstalled. pyenv will
be installed automatically the first time it is needed and can be updated
later. This module will *not* automatically install packages which pyenv
will need to compile the versions of python.

If pyenv is run as the root user then it will be installed to /usr/local/pyenv,
otherwise it will be installed to the users ~/.pyenv directory. To make
pyenv available in the shell you may need to add the pyenv/shims and pyenv/bin
directories to the users PATH. If you are installing as root and want other
users to be able to access pyenv then you will need to add pyenv_ROOT to
their environment.

This is how a state configuration could look like:

.. code-block:: yaml

    pyenv-deps:
      pkg.installed:
        - pkgs:
          - make
          - build-essential
          - libssl-dev
          - zlib1g-dev
          - libbz2-dev
          - libreadline-dev
          - libsqlite3-dev
          - wget
          - curl
          - llvm
    python-2.6:
      pyenv.absent:
        - require:
          - pkg: pyenv-deps

    python-2.7.6:
      pyenv.installed:
        - default: True
        - require:
          - pkg: pyenv-deps
'''
from __future__ import absolute_import

# Import python libs
import re


def _check_pyenv(ret, user=None):
    '''
    Check to see if pyenv is installed.
    '''
    if not __salt__['pyenv.is_installed'](user):
        ret['result'] = False
        ret['comment'] = 'pyenv is not installed.'
    return ret


def _python_installed(ret, python, user=None):
    '''
    Check to see if given python is installed.
    '''
    default = __salt__['pyenv.default'](runas=user)
    for version in __salt__['pyenv.versions'](user):
        if version == python:
            ret['result'] = True
            ret['comment'] = 'Requested python exists.'
            ret['default'] = default == python
            break

    return ret


def _check_and_install_python(ret, python, default=False, user=None):
    '''
    Verify that python is installed, install if unavailable
    '''
    ret = _python_installed(ret, python, user=user)
    if not ret['result']:
        if __salt__['pyenv.install_python'](python, runas=user):
            ret['result'] = True
            ret['changes'][python] = 'Installed'
            ret['comment'] = 'Successfully installed python'
            ret['default'] = default
        else:
            ret['result'] = False
            ret['comment'] = 'Could not install python.'
            return ret

    if default:
        __salt__['pyenv.default'](python, runas=user)

    return ret


def installed(name, default=False, user=None):
    '''
    Verify that the specified python is installed with pyenv. pyenv is
    installed if necessary.

    name
        The version of python to install

    default : False
        Whether to make this python the default.

    user: None
        The user to run pyenv as.

        .. versionadded:: 0.17.0

    .. versionadded:: 0.16.0
    '''
    ret = {'name': name, 'result': None, 'comment': '', 'changes': {}}

    if name.startswith('python-'):
        name = re.sub(r'^python-', '', name)

    if __opts__['test']:
        ret['comment'] = 'python {0} is set to be installed'.format(name)
        return ret

    ret = _check_pyenv(ret, user)
    if ret['result'] is False:
        if not __salt__['pyenv.install'](user):
            ret['comment'] = 'pyenv failed to install'
            return ret
        else:
            return _check_and_install_python(ret, name, default, user=user)
    else:
        return _check_and_install_python(ret, name, default, user=user)


def _check_and_uninstall_python(ret, python, user=None):
    '''
    Verify that python is uninstalled
    '''
    ret = _python_installed(ret, python, user=user)
    if ret['result']:
        if ret['default']:
            __salt__['pyenv.default']('system', runas=user)

        if __salt__['pyenv.uninstall_python'](python, runas=user):
            ret['result'] = True
            ret['changes'][python] = 'Uninstalled'
            ret['comment'] = 'Successfully removed python'
            return ret
        else:
            ret['result'] = False
            ret['comment'] = 'Failed to uninstall python'
            return ret
    else:
        ret['result'] = True
        ret['comment'] = 'python {0} is already absent'.format(python)

    return ret


def absent(name, user=None):
    '''
    Verify that the specified python is not installed with pyenv. pyenv
    is installed if necessary.

    name
        The version of python to uninstall

    user: None
        The user to run pyenv as.

        .. versionadded:: 0.17.0

    .. versionadded:: 0.16.0
    '''
    ret = {'name': name, 'result': None, 'comment': '', 'changes': {}}

    if name.startswith('python-'):
        name = re.sub(r'^python-', '', name)

    if __opts__['test']:
        ret['comment'] = 'python {0} is set to be uninstalled'.format(name)
        return ret

    ret = _check_pyenv(ret, user)
    if ret['result'] is False:
        ret['result'] = True
        ret['comment'] = 'pyenv not installed, {0} not either'.format(name)
        return ret
    else:
        return _check_and_uninstall_python(ret, name, user=user)


def install_pyenv(name, user=None):
    '''
    Install pyenv if not installed. Allows you to require pyenv be installed
    prior to installing the plugins. Useful if you want to install pyenv
    plugins via the git or file modules and need them installed before
    installing any rubies.

    Use the pyenv.root configuration option to set the path for pyenv if you
    want a system wide install that is not in a user home dir.

    user: None
        The user to run pyenv as.
    '''
    ret = {'name': name, 'result': None, 'comment': '', 'changes': {}}

    if __opts__['test']:
        ret['comment'] = 'pyenv is set to be installed'
        return ret

    return _check_and_install_python(ret, user)



# -*- coding: utf-8 -*-
'''
Management of addresses and names in hosts file
===============================================

The ``/etc/hosts`` file can be managed to contain definitions for specific hosts:

.. code-block:: yaml

    salt-master:
      host.present:
        - ip: 192.168.0.42

Or using the ``names`` directive, you can put several names for the same IP.
(Do not try one name with space-separated values).

.. code-block:: yaml

    server1:
      host.present:
        - ip: 192.168.0.42
        - names:
          - server1
          - florida

.. note::

    Changing the ``names`` in ``host.present`` does not cause an
    update to remove the old entry.

.. code-block:: yaml

    server1:
      host.present:
        - ip:
          - 192.168.0.42
          - 192.168.0.43
          - 192.168.0.44
        - names:
          - server1

'''
from __future__ import absolute_import

import salt.utils.validate.net


def present(name, ip):  # pylint: disable=C0103
    '''
    Ensures that the named host is present with the given ip

    name
        The host to assign an ip to

    ip
        The ip addr(s) to apply to the host
    '''
    ret = {'name': name,
           'changes': {},
           'result': None,
           'comment': ''}

    if not isinstance(ip, list):
        ip = [ip]

    comments = []
    for _ip in ip:
        if __salt__['hosts.has_pair'](_ip, name):
            ret['result'] = True
            comments.append('Host {0} ({1}) already present'.format(name, _ip))
        else:
            if __opts__['test']:
                comments.append('Host {0} ({1}) needs to be added/updated'.format(name, _ip))
            else:
                if salt.utils.validate.net.ipv4_addr(_ip) or salt.utils.validate.net.ipv6_addr(_ip):
                    if __salt__['hosts.add_host'](_ip, name):
                        ret['changes'] = {'host': name}
                        ret['result'] = True
                        comments.append('Added host {0} ({1})'.format(name, _ip))
                    else:
                        ret['result'] = False
                        comments.append('Failed to set host')
                else:
                    ret['result'] = False
                    comments.append('Invalid IP Address for {0} ({1})'.format(name, _ip))
    ret['comment'] = '\n'.join(comments)
    return ret


def absent(name, ip):  # pylint: disable=C0103
    '''
    Ensure that the named host is absent

    name
        The host to remove

    ip
        The ip addr(s) of the host to remove
    '''
    ret = {'name': name,
           'changes': {},
           'result': None,
           'comment': ''}

    if not isinstance(ip, list):
        ip = [ip]

    comments = []
    for _ip in ip:
        if not __salt__['hosts.has_pair'](_ip, name):
            ret['result'] = True
            comments.append('Host {0} ({1}) already absent'.format(name, _ip))
        else:
            if __opts__['test']:
                comments.append('Host {0} ({1} needs to be removed'.format(name, _ip))
            else:
                if __salt__['hosts.rm_host'](_ip, name):
                    ret['changes'] = {'host': name}
                    ret['result'] = True
                    comments.append('Removed host {0} ({1})'.format(name, _ip))
                else:
                    ret['result'] = False
                    comments.append('Failed to remove host')
    ret['comment'] = '\n'.join(comments)
    return ret



# -*- coding: utf-8 -*-
'''
Module for Firing Events via PagerDuty

.. versionadded:: 2014.1.0

:configuration: This module can be used by specifying the name of a
    configuration profile in the minion config, minion pillar, or master
    config.

    For example:

    .. code-block:: yaml

        my-pagerduty-account:
            pagerduty.api_key: F3Rbyjbve43rfFWf2214
            pagerduty.subdomain: mysubdomain
'''
from __future__ import absolute_import

# Import python libs
import yaml
import json

# Import salt libs
import salt.utils.pagerduty
from salt.ext.six import string_types


def __virtual__():
    '''
    No dependencies outside of what Salt itself requires
    '''
    return True


def list_services(profile=None, api_key=None):
    '''
    List services belonging to this account

    CLI Example:

        salt myminion pagerduty.list_services my-pagerduty-account
    '''
    return salt.utils.pagerduty.list_items(
        'services',
        'name',
        __salt__['config.option'](profile),
        api_key,
        opts=__opts__
    )


def list_incidents(profile=None, api_key=None):
    '''
    List incidents belonging to this account

    CLI Example:

        salt myminion pagerduty.list_incidents my-pagerduty-account
    '''
    return salt.utils.pagerduty.list_items(
        'incidents',
        'id',
        __salt__['config.option'](profile),
        api_key,
        opts=__opts__
    )


def list_users(profile=None, api_key=None):
    '''
    List users belonging to this account

    CLI Example:

        salt myminion pagerduty.list_users my-pagerduty-account
    '''
    return salt.utils.pagerduty.list_items(
        'users',
        'id',
        __salt__['config.option'](profile),
        api_key,
        opts=__opts__
    )


def list_schedules(profile=None, api_key=None):
    '''
    List schedules belonging to this account

    CLI Example:

        salt myminion pagerduty.list_schedules my-pagerduty-account
    '''
    return salt.utils.pagerduty.list_items(
        'schedules',
        'id',
        __salt__['config.option'](profile),
        api_key,
        opts=__opts__
    )


def list_windows(profile=None, api_key=None):
    '''
    List maintenance windows belonging to this account

    CLI Example:

        salt myminion pagerduty.list_windows my-pagerduty-account
        salt myminion pagerduty.list_maintenance_windows my-pagerduty-account
    '''
    return salt.utils.pagerduty.list_items(
        'maintenance_windows',
        'id',
        __salt__['config.option'](profile),
        api_key,
        opts=__opts__
    )


# The long version, added for consistency
list_maintenance_windows = salt.utils.alias_function(list_windows, 'list_maintenance_windows')


def list_policies(profile=None, api_key=None):
    '''
    List escalation policies belonging to this account

    CLI Example:

        salt myminion pagerduty.list_policies my-pagerduty-account
        salt myminion pagerduty.list_escalation_policies my-pagerduty-account
    '''
    return salt.utils.pagerduty.list_items(
        'escalation_policies',
        'id',
        __salt__['config.option'](profile),
        api_key,
        opts=__opts__
    )


# The long version, added for consistency
list_escalation_policies = salt.utils.alias_function(list_policies, 'list_escalation_policies')


def create_event(service_key=None, description=None, details=None,
                 incident_key=None, profile=None):
    '''
    Create an event in PagerDuty. Designed for use in states.

    CLI Example:

    .. code-block:: yaml

        salt myminion pagerduty.create_event <service_key> <description> <details> \
        profile=my-pagerduty-account

    The following parameters are required:

    service_key
        This key can be found by using pagerduty.list_services.

    description
        This is a short description of the event.

    details
        This can be a more detailed description of the event.

    profile
        This refers to the configuration profile to use to connect to the
        PagerDuty service.
    '''
    trigger_url = 'https://events.pagerduty.com/generic/2010-04-15/create_event.json'

    if isinstance(details, string_types):
        details = yaml.safe_load(details)
        if isinstance(details, string_types):
            details = {'details': details}

    ret = json.loads(salt.utils.pagerduty.query(
        method='POST',
        profile_dict=__salt__['config.option'](profile),
        api_key=service_key,
        data={
            'service_key': service_key,
            'incident_key': incident_key,
            'event_type': 'trigger',
            'description': description,
            'details': details,
        },
        url=trigger_url,
        opts=__opts__
    ))
    return ret



# -*- coding: utf-8 -*-
'''
    :codeauthor: :email:`Rupesh Tare <rupesht@saltstack.com>`
'''

# Import Python Libs
from __future__ import absolute_import

# Import Salt Testing Libs
from salttesting import TestCase, skipIf
from salttesting.mock import (
    MagicMock,
    patch,
    NO_MOCK,
    NO_MOCK_REASON
)
from salttesting.helpers import ensure_in_syspath

ensure_in_syspath('../../')

# Import Salt Libs
from salt.modules import monit


# Globals
monit.__salt__ = {}


@skipIf(NO_MOCK, NO_MOCK_REASON)
class MonitTestCase(TestCase):
    '''
    Test cases for salt.modules.aptpkg
    '''
    def test_start(self):
        '''
        Test for start
        '''
        with patch.dict(monit.__salt__,
                        {'cmd.retcode': MagicMock(return_value=False)}):
            self.assertTrue(monit.start('name'))

    def test_stop(self):
        '''
        Test for Stops service via monit
        '''
        with patch.dict(monit.__salt__,
                        {'cmd.retcode': MagicMock(return_value=False)}):
            self.assertTrue(monit.stop('name'))

    def test_restart(self):
        '''
        Test for Restart service via monit
        '''
        with patch.dict(monit.__salt__,
                        {'cmd.retcode': MagicMock(return_value=False)}):
            self.assertTrue(monit.restart('name'))

    def test_unmonitor(self):
        '''
        Test for Unmonitor service via monit
        '''
        with patch.dict(monit.__salt__,
                        {'cmd.retcode': MagicMock(return_value=False)}):
            self.assertTrue(monit.unmonitor('name'))

    def test_monitor(self):
        '''
        Test for monitor service via monit
        '''
        with patch.dict(monit.__salt__,
                        {'cmd.retcode': MagicMock(return_value=False)}):
            self.assertTrue(monit.monitor('name'))

    def test_summary(self):
        '''
        Test for Display a summary from monit
        '''
        mock = MagicMock(side_effect=['daemon is not running',
                                      'A\nB\nC\nD\nE'])
        with patch.dict(monit.__salt__, {'cmd.run': mock}):
            self.assertEqual(monit.summary(),
                             {'monit': 'daemon is not running',
                              'result': False})

            self.assertEqual(monit.summary(), {})

    def test_status(self):
        '''
        Test for Display a process status from monit
        '''
        with patch.dict(monit.__salt__,
                        {'cmd.run':
                         MagicMock(return_value='Process')}):
            self.assertEqual(monit.status('service'), 'No such service')


if __name__ == '__main__':
    from integration import run_tests
    run_tests(MonitTestCase, needs_daemon=False)



# -*- coding: utf-8 -*-
'''
Renderer that will decrypt GPG ciphers

Any key in the SLS file can be a GPG cipher, and this renderer will decrypt it
before passing it off to Salt. This allows you to safely store secrets in
source control, in such a way that only your Salt master can decrypt them and
distribute them only to the minions that need them.

The typical use-case would be to use ciphers in your pillar data, and keep a
secret key on your master. You can put the public key in source control so that
developers can add new secrets quickly and easily.

This renderer requires the gpg binary.

**No python libraries are required as of the 2015.8.3 release.**

To set things up, you will first need to generate a keypair. On your master,
run:

.. code-block:: shell

    # mkdir -p /etc/salt/gpgkeys
    # chmod 0700 /etc/salt/gpgkeys
    # gpg --gen-key --homedir /etc/salt/gpgkeys

Do not supply a password for your keypair, and use a name that makes sense
for your application. Be sure to back up your gpg directory someplace safe!

.. note::
    Unfortunately, there are some scenarios - for example, on virtual machines
    which don’t have real hardware - where insufficient entropy causes key
    generation to be extremely slow. If you come across this problem, you should
    investigate means of increasing the system entropy. On virtualised Linux
    systems, this can often be achieved by installing the rng-tools package.

To retrieve the public key:

.. code-block:: shell

    # gpg --homedir /etc/salt/gpgkeys --armor --export <KEY-NAME> \
        > exported_pubkey.gpg

Now, to encrypt secrets, copy the public key to your local machine and run:

.. code-block:: shell

    $ gpg --import exported_pubkey.gpg

To generate a cipher from a secret:

.. code-block:: shell

   $ echo -n "supersecret" | gpg --armor --encrypt -r <KEY-name>

To apply the renderer on a file-by-file basis add the following line to the
top of any pillar with gpg data in it:

    .. code-block:: yaml

        #!yaml|gpg

Now with your renderer configured, you can include your ciphers in your pillar data like so:

.. code-block:: yaml

    #!yaml|gpg

    a-secret: |
      -----BEGIN PGP MESSAGE-----
      Version: GnuPG v1

      hQEMAweRHKaPCfNeAQf9GLTN16hCfXAbPwU6BbBK0unOc7i9/etGuVc5CyU9Q6um
      QuetdvQVLFO/HkrC4lgeNQdM6D9E8PKonMlgJPyUvC8ggxhj0/IPFEKmrsnv2k6+
      cnEfmVexS7o/U1VOVjoyUeliMCJlAz/30RXaME49Cpi6No2+vKD8a4q4nZN1UZcG
      RhkhC0S22zNxOXQ38TBkmtJcqxnqT6YWKTUsjVubW3bVC+u2HGqJHu79wmwuN8tz
      m4wBkfCAd8Eyo2jEnWQcM4TcXiF01XPL4z4g1/9AAxh+Q4d8RIRP4fbw7ct4nCJv
      Gr9v2DTF7HNigIMl4ivMIn9fp+EZurJNiQskLgNbktJGAeEKYkqX5iCuB1b693hJ
      FKlwHiJt5yA8X2dDtfk8/Ph1Jx2TwGS+lGjlZaNqp3R1xuAZzXzZMLyZDe5+i3RJ
      skqmFTbOiA==
      =Eqsm
      -----END PGP MESSAGE-----
'''

# Import python libs
from __future__ import absolute_import
import os
import re
import logging
from subprocess import Popen, PIPE

# Import salt libs
import salt.utils
import salt.syspaths
from salt.exceptions import SaltRenderError

# Import 3rd-party libs
import salt.ext.six as six

if salt.utils.which('gpg'):
    HAS_GPG = True
else:
    HAS_GPG = False
    raise SaltRenderError('GPG unavailable')

LOG = logging.getLogger(__name__)

GPG_HEADER = re.compile(r'-----BEGIN PGP MESSAGE-----')


def _get_gpg_exec():
    '''
    return the GPG executable or raise an error
    '''
    gpg_exec = salt.utils.which('gpg')
    if gpg_exec:
        return gpg_exec
    else:
        raise SaltRenderError('GPG unavailable')


def _get_key_dir():
    '''
    return the location of the GPG key directory
    '''
    key_dir = __salt__['config.get']('gpg_keydir')
    if not key_dir:
        key_dir = os.path.join(
            __salt__['config.get']('config_dir'),
            'gpgkeys'
        )

    return key_dir


def _decrypt_ciphertext(cipher):
    '''
    Given a block of ciphertext as a string, and a gpg object, try to decrypt
    the cipher and return the decrypted string. If the cipher cannot be
    decrypted, log the error, and return the ciphertext back out.
    '''
    cmd = [_get_gpg_exec(), '--homedir', _get_key_dir(), '-d']
    proc = Popen(cmd, stdin=PIPE, stdout=PIPE, stderr=PIPE, shell=False)
    decrypted_data, decrypt_error = proc.communicate(input=cipher)
    if not decrypted_data:
        LOG.error('Could not decrypt cipher %s, received: %s', cipher, decrypt_error)
        return cipher
    else:
        return str(decrypted_data)


def _decrypt_object(obj):
    '''
    Recursively try to decrypt any object. If the object is a string, and
    it contains a valid GPG header, decrypt it, otherwise keep going until
    a string is found.
    '''
    if isinstance(obj, str):
        if GPG_HEADER.search(obj):
            return _decrypt_ciphertext(obj)
        else:
            return obj
    elif isinstance(obj, dict):
        for key, val in six.iteritems(obj):
            obj[key] = _decrypt_object(val)
        return obj
    elif isinstance(obj, list):
        for key, value in enumerate(obj):
            obj[key] = _decrypt_object(value)
        return obj
    else:
        return obj


def render(gpg_data, saltenv='base', sls='', argline='', **kwargs):
    '''
    Create a gpg object given a gpg_keydir, and then use it to try to decrypt
    the data to be rendered.
    '''
    if not _get_gpg_exec():
        raise SaltRenderError('GPG unavailable')
    LOG.debug('Reading GPG keys from: %s', _get_key_dir())

    return _decrypt_object(gpg_data)



# -*- coding: utf-8 -*-
'''
    :codeauthor: :email:`Jayesh Kariya <jayeshk@saltstack.com>`
'''
# Import Python libs
from __future__ import absolute_import

# Import Salt Testing Libs
from salttesting import skipIf, TestCase
from salttesting.mock import (
    NO_MOCK,
    NO_MOCK_REASON,
    MagicMock,
    patch)

from salttesting.helpers import ensure_in_syspath

ensure_in_syspath('../../')

# Import Salt Libs
from salt.modules import powerpath

# Globals
powerpath.__salt__ = {}


@skipIf(NO_MOCK, NO_MOCK_REASON)
class PowerpathTestCase(TestCase):
    '''
    Test cases for salt.modules.powerpath
    '''
    @patch('os.path.exists')
    def test_has_powerpath(self, mock_exists):
        '''
        Test for powerpath
        '''
        mock_exists.return_value = True
        self.assertTrue(powerpath.has_powerpath())

        mock_exists.return_value = False
        self.assertFalse(powerpath.has_powerpath())

    def test_list_licenses(self):
        '''
        Test to returns a list of applied powerpath license keys
        '''
        with patch.dict(powerpath.__salt__,
                        {'cmd.run': MagicMock(return_value='A\nB')}):
            self.assertListEqual(powerpath.list_licenses(), [])

    def test_add_license(self):
        '''
        Test to add a license
        '''
        with patch.object(powerpath, 'has_powerpath', return_value=False):
            self.assertDictEqual(powerpath.add_license('key'),
                                 {'output': 'PowerPath is not installed',
                                  'result': False, 'retcode': -1})

        mock = MagicMock(return_value={'retcode': 1, 'stderr': 'stderr'})
        with patch.object(powerpath, 'has_powerpath', return_value=True):
            with patch.dict(powerpath.__salt__, {'cmd.run_all': mock}):
                self.assertDictEqual(powerpath.add_license('key'),
                                     {'output': 'stderr', 'result': False,
                                      'retcode': 1})

    def test_remove_license(self):
        '''
        Test to remove a license
        '''
        with patch.object(powerpath, 'has_powerpath', return_value=False):
            self.assertDictEqual(powerpath.remove_license('key'),
                                 {'output': 'PowerPath is not installed',
                                  'result': False, 'retcode': -1})

        mock = MagicMock(return_value={'retcode': 1, 'stderr': 'stderr'})
        with patch.object(powerpath, 'has_powerpath', return_value=True):
            with patch.dict(powerpath.__salt__, {'cmd.run_all': mock}):
                self.assertDictEqual(powerpath.remove_license('key'),
                                     {'output': 'stderr', 'result': False,
                                      'retcode': 1})


if __name__ == '__main__':
    from integration import run_tests
    run_tests(PowerpathTestCase, needs_daemon=False)



# -*- coding: utf-8 -*-
'''
Tests for the service state
'''
# Import python libs
from __future__ import absolute_import
import os

# Import Salt Testing libs
from salttesting import skipIf
from salttesting.helpers import ensure_in_syspath, destructiveTest
ensure_in_syspath('../../')

# Import salt libs
from integration import ModuleCase, SaltReturnAssertsMixIn


# used to cache the status of whether the package was installed for quick test
# skipping if it was not
PKG_INSTALLED = None


def installed(run_function, package):
    '''
    Install apache prior to running tests so that they may be skipped if the
    package fails to download
    '''
    global PKG_INSTALLED
    if not PKG_INSTALLED:
        run_function('state.single', ['pkg.installed', package])
        PKG_INSTALLED = True if run_function('pkg.version', [package]) else False
    return PKG_INSTALLED


@destructiveTest
@skipIf(os.geteuid() != 0, 'You must be logged in as root to run this test')
class ServiceTest(ModuleCase,
                  SaltReturnAssertsMixIn):
    '''
    Test service state module
    '''
    def setUp(self):
        '''
        Setup package and service names
        '''
        # Taken from https://github.com/saltstack-formulas/apache-formula/blob/master/apache/map.jinja
        os_family = self.run_function('grains.get', ['os_family'])

        if os_family == 'Debian':
            self.package = 'apache2'
            self.service = 'apache2'
        elif os_family == 'RedHat':
            self.package = 'httpd'
            self.service = 'httpd'
        elif os_family == 'Suse':
            self.package = 'apache2'
            self.service = 'apache2'
        elif os_family == 'FreeBSD':
            self.package = 'apache22'
            self.service = 'apache22'
        elif os_family == 'MacOS':
            self.package = 'homebrew/apache/httpd24'
            self.service = 'org.apache.httpd'
        else:
            self.skipTest('This platform, {0}, has not yet been configured'
                          ' to run this test'.format(os_family))

    def test_aaa_setUp_package(self):
        '''
        Install apache package
        '''
        installed(self.run_function, self.package)

    def test_zzz_tearDown_package(self):
        '''
        Remove apache package
        '''
        # Skip if package was not installed
        if not PKG_INSTALLED:
            self.skipTest('Package containing service used by test was not installed')

        self.run_function('state.single', ['pkg.removed', self.package])

    def test_running(self):
        '''
        Test service.running
        '''
        # Skip test if package was not installed
        if not PKG_INSTALLED:
            self.skipTest('Package containing service used by test was not installed')

        # Configure state parameters
        state_params = [
            'service.running',
            'name={0}'.format(self.service),
        ]

        # Setup initial state, run base test and inductive test
        self.run_function('state.single', ['service.dead', self.service])
        start_ret = self.run_function('state.single', state_params)
        already_ret = self.run_function('state.single', state_params)

        # Validate base state
        self.assertSaltTrueReturn(start_ret)
        self.assertInSaltComment('Started Service {0}'.format(self.service), start_ret)
        self.assertSaltStateChangesEqual(start_ret, {self.service: True})

        # Validate inductive state
        self.assertSaltTrueReturn(already_ret)
        self.assertInSaltComment('The service {0} is already running'.format(self.service), already_ret)
        self.assertSaltStateChangesEqual(already_ret, {})

    def test_running_test(self):
        '''
        Test service.running with test=True
        '''
        # Skip test if package was not installed
        if not PKG_INSTALLED:
            self.skipTest('Package containing service used by test was not installed')

        # Configure state parameters
        state_params = [
            'service.running',
            'name={0}'.format(self.service),
            'test=True',
        ]

        # apply the state in test mode
        self.run_function('state.single', ['service.dead', self.service])
        test_ret = self.run_function('state.single', state_params)
        # Validate test state
        self.assertSaltNoneReturn(test_ret)
        self.assertInSaltComment('Service {0} is set to start'.format(self.service), test_ret)
        self.assertSaltStateChangesEqual(test_ret, {})

    def test_running_enabled(self):
        '''
        Test service.running with enable=True
        '''
        # Skip test if package was not installed
        if not PKG_INSTALLED:
            self.skipTest('Package containing service used by test was not installed')

        # Skip test if functionality not present on system
        for func in 'enable', 'enabled':
            if not self.run_function('sys.doc', ['service.{0}'.format(func)]):
                self.skipTest('service.{0} function is not available on this system'.format(func))

        # Configure state parameters
        enabled_params = [
            'service.running',
            'name={0}'.format(self.service),
            'enable=True',
        ]

        # Apply state with enable parameter True
        self.run_function('state.single', ['service.dead', self.service, 'enable=False'])
        enabled_ret = self.run_function('state.single', enabled_params)
        # Validate enabled state
        self.assertSaltTrueReturn(enabled_ret)
        self.assertInSaltComment('enabled', enabled_ret)
        self.assertSaltStateChangesEqual(enabled_ret, {self.service: True})

        # Apply state a second time with enable parameter True
        self.run_function('state.single', ['service.dead', self.service])
        already_enabled_ret = self.run_function('state.single', enabled_params)
        # Validate enabled state
        self.assertSaltTrueReturn(already_enabled_ret)
        self.assertInSaltComment('already enabled', already_enabled_ret)
        self.assertSaltStateChangesEqual(already_enabled_ret, {self.service: True})

    def test_running_disabled(self):
        '''
        Test service.running with enable=False
        '''
        # Skip test if package was not installed
        if not PKG_INSTALLED:
            self.skipTest('Package containing service used by test was not installed')

        # Skip test if functionality not present on system
        for func in 'disable', 'disabled':
            if not self.run_function('sys.doc', ['service.{0}'.format(func)]):
                self.skipTest('service.{0} function is not available on this system'.format(func))

        # Configure state parameters
        disabled_params = [
            'service.running',
            'name={0}'.format(self.service),
            'enable=False',
        ]

        # Apply state with enable parameter False
        self.run_function('state.single', ['service.dead', self.service, 'enable=True'])
        disabled_ret = self.run_function('state.single', disabled_params)
        # Validate disabled state
        self.assertSaltTrueReturn(disabled_ret)
        self.assertInSaltComment('disabled', disabled_ret)
        self.assertSaltStateChangesEqual(disabled_ret, {self.service: True})

        # Apply state a second time with enable parameter False
        self.run_function('state.single', ['service.dead', self.service])
        already_disabled_ret = self.run_function('state.single', disabled_params)
        self.assertSaltStateChangesEqual(disabled_ret, {self.service: True})
        # Validate disabled state
        self.assertSaltTrueReturn(already_disabled_ret)
        self.assertInSaltComment('already disabled', already_disabled_ret)
        self.assertSaltStateChangesEqual(already_disabled_ret, {self.service: True})

    def test_running_delayed(self):
        '''
        Test service.running with delay
        '''
        # Skip test if package was not installed
        if not PKG_INSTALLED:
            self.skipTest('Package containing service used by test was not installed')

        # Configure state parameters
        init_delay = 6.2832
        delayed_params = [
            'service.running',
            'name={0}'.format(self.service),
            'init_delay={0}'.format(init_delay)
        ]
        state_key = 'service_|-{0}_|-{0}_|-running'.format(self.service)

        # Apply state with init_delay parameter
        self.run_function('state.single', ['service.dead', self.service])
        delayed_ret = self.run_function('state.single', delayed_params)
        # Validate delayed state
        self.assertSaltTrueReturn(delayed_ret)
        self.assertInSaltComment('Delayed return for {0} seconds'.format(init_delay), delayed_ret)
        self.assertSaltStateChangesEqual(delayed_ret, {self.service: True})
        self.assertTrue(delayed_ret[state_key]['duration']/1000 >= init_delay)

    def test_dead(self):
        '''
        Test service.dead
        '''
        # Skip test if package was not installed
        if not PKG_INSTALLED:
            self.skipTest('Package containing service used by test was not installed')

        # Configure state parameters
        state_params = [
            'service.dead',
            'name={0}'.format(self.service),
        ]

        # Setup initial state, run base test and inductive test
        self.run_function('state.single', ['service.running', self.service, 'init_delay=3'])
        kill_ret = self.run_function('state.single', state_params)
        already_ret = self.run_function('state.single', state_params)

        # Validate base state
        self.assertSaltTrueReturn(kill_ret)
        self.assertInSaltComment('Service {0} was killed'.format(self.service), kill_ret)
        self.assertSaltStateChangesEqual(kill_ret, {self.service: True})

        # Validate inductive state
        self.assertSaltTrueReturn(already_ret)
        self.assertInSaltComment('The service {0} is already dead'.format(self.service), already_ret)
        self.assertSaltStateChangesEqual(already_ret, {})

    def test_dead_test(self):
        '''
        Test service.dead with test=True
        '''
        # Skip test if package was not installed
        if not PKG_INSTALLED:
            self.skipTest('Package containing service used by test was not installed')

        # Configure state parameters
        state_params = [
            'service.dead',
            'name={0}'.format(self.service),
            'test=True',
        ]

        # apply the state in test mode
        self.run_function('state.single', ['service.running', self.service, 'init_delay=3'])
        test_ret = self.run_function('state.single', state_params)
        # Validate base state
        self.assertSaltNoneReturn(test_ret)
        self.assertInSaltComment('Service {0} is set to be killed'.format(self.service), test_ret)
        self.assertSaltStateChangesEqual(test_ret, {})

    def test_dead_enabled(self):
        '''
        Test service.dead with enable=True
        '''
        # Skip test if package was not installed
        if not PKG_INSTALLED:
            self.skipTest('Package containing service used by test was not installed')

        # Skip test if functionality not present on system
        for func in 'enable', 'enabled':
            if not self.run_function('sys.doc', ['service.{0}'.format(func)]):
                self.skipTest('service.{0} function is not available on this system'.format(func))

        # Configure state parameters
        enabled_params = [
            'service.dead',
            'name={0}'.format(self.service),
            'enable=True',
        ]

        # Apply state with enable parameter True
        self.run_function('state.single', ['service.running', self.service, 'enable=False', 'init_delay=3'])
        enabled_ret = self.run_function('state.single', enabled_params)
        # Validate enabled state
        self.assertSaltTrueReturn(enabled_ret)
        self.assertInSaltComment('enabled', enabled_ret)
        self.assertSaltStateChangesEqual(enabled_ret, {self.service: True})

        # Apply state a second time with enable parameter True
        self.run_function('state.single', ['service.running', self.service, 'init_delay=3'])
        already_enabled_ret = self.run_function('state.single', enabled_params)
        # Validate enabled state
        self.assertSaltTrueReturn(already_enabled_ret)
        self.assertInSaltComment('already enabled', already_enabled_ret)
        self.assertSaltStateChangesEqual(already_enabled_ret, {self.service: True})

    def test_dead_disabled(self):
        '''
        Test service.dead with enable=False
        '''
        # Skip test if package was not installed
        if not PKG_INSTALLED:
            self.skipTest('Package containing service used by test was not installed')

        # Skip test if functionality not present on system
        for func in 'disable', 'disabled':
            if not self.run_function('sys.doc', ['service.{0}'.format(func)]):
                self.skipTest('service.{0} function is not available on this system'.format(func))

        # Configure state parameters
        disabled_params = [
            'service.dead',
            'name={0}'.format(self.service),
            'enable=False',
        ]

        # Apply state with enable parameter False
        self.run_function('state.single', ['service.running', self.service, 'enable=True', 'init_delay=3'])
        disabled_ret = self.run_function('state.single', disabled_params)
        # Validate disabled state
        self.assertSaltTrueReturn(disabled_ret)
        self.assertInSaltComment('disabled', disabled_ret)
        self.assertSaltStateChangesEqual(disabled_ret, {self.service: True})

        # Apply state a second time with enable parameter False
        self.run_function('state.single', ['service.running', self.service, 'init_delay=3'])
        already_disabled_ret = self.run_function('state.single', disabled_params)
        self.assertSaltStateChangesEqual(disabled_ret, {self.service: True})
        # Validate disabled state
        self.assertSaltTrueReturn(already_disabled_ret)
        self.assertInSaltComment('already disabled', already_disabled_ret)
        self.assertSaltStateChangesEqual(already_disabled_ret, {self.service: True})

    def test_enabled(self):
        '''
        Test service.enabled
        '''
        # Skip test if package was not installed
        if not PKG_INSTALLED:
            self.skipTest('Package containing service used by test was not installed')

        # Skip test if functionality not present on system
        for func in 'enable', 'enabled':
            if not self.run_function('sys.doc', ['service.{0}'.format(func)]):
                self.skipTest('service.{0} function is not available on this system'.format(func))

        # Configure state parameters
        state_params = [
            'service.enabled',
            'name={0}'.format(self.service),
        ]

        # Apply state
        self.run_function('state.single', ['service.disabled', self.service])
        enabled_ret = self.run_function('state.single', state_params)
        # Validate enabled state
        self.assertSaltTrueReturn(enabled_ret)
        self.assertInSaltComment('enabled', enabled_ret)
        self.assertSaltStateChangesEqual(enabled_ret, {self.service: True})

        # Apply state a second time
        already_enabled_ret = self.run_function('state.single', state_params)
        # Validate enabled state
        self.assertSaltTrueReturn(already_enabled_ret)
        self.assertInSaltComment('already enabled', already_enabled_ret)
        self.assertSaltStateChangesEqual(already_enabled_ret, {})

    def test_enabled_test(self):
        '''
        Test service.enabled with test=True
        '''
        # Skip test if package was not installed
        if not PKG_INSTALLED:
            self.skipTest('Package containing service used by test was not installed')

        # Skip test if functionality not present on system
        for func in 'enable', 'enabled':
            if not self.run_function('sys.doc', ['service.{0}'.format(func)]):
                self.skipTest('service.{0} function is not available on this system'.format(func))

        # Configure state parameters
        state_params = [
            'service.enabled',
            'name={0}'.format(self.service),
            'test=True',
        ]

        # apply the state in test mode
        self.run_function('state.single', ['service.disabled', self.service])
        test_ret = self.run_function('state.single', state_params)
        # Validate base state
        self.assertSaltNoneReturn(test_ret)
        self.assertInSaltComment('Service {0} set to be enabled'.format(self.service), test_ret)
        self.assertSaltStateChangesEqual(test_ret, {})

    def test_disabled(self):
        '''
        Test service.disabled
        '''
        # Skip test if package was not installed
        if not PKG_INSTALLED:
            self.skipTest('Package containing service used by test was not installed')

        # Skip test if functionality not present on system
        for func in 'disable', 'disabled':
            if not self.run_function('sys.doc', ['service.{0}'.format(func)]):
                self.skipTest('service.{0} function is not available on this system'.format(func))

        # Configure state parameters
        state_params = [
            'service.disabled',
            'name={0}'.format(self.service),
        ]

        # Apply state
        self.run_function('state.single', ['service.enabled', self.service])
        disabled_ret = self.run_function('state.single', state_params)
        # Validate disabled state
        self.assertSaltTrueReturn(disabled_ret)
        self.assertInSaltComment('disabled', disabled_ret)
        self.assertSaltStateChangesEqual(disabled_ret, {self.service: True})

        # Apply state a second time
        already_disabled_ret = self.run_function('state.single', state_params)
        # Validate disabled state
        self.assertSaltTrueReturn(already_disabled_ret)
        self.assertInSaltComment('already disabled', already_disabled_ret)
        self.assertSaltStateChangesEqual(already_disabled_ret, {})

    def test_disabled_test(self):
        '''
        Test service.disabled with test=True
        '''
        # Skip test if package was not installed
        if not PKG_INSTALLED:
            self.skipTest('Package containing service used by test was not installed')

        # Skip test if functionality not present on system
        for func in 'disable', 'disabled':
            if not self.run_function('sys.doc', ['service.{0}'.format(func)]):
                self.skipTest('service.{0} function is not available on this system'.format(func))

        # Configure state parameters
        state_params = [
            'service.disabled',
            'name={0}'.format(self.service),
            'test=True',
        ]

        # apply the state in test mode
        self.run_function('state.single', ['service.enabled', self.service])
        test_ret = self.run_function('state.single', state_params)
        # Validate base state
        self.assertSaltNoneReturn(test_ret)
        self.assertInSaltComment('Service {0} set to be disabled'.format(self.service), test_ret)
        self.assertSaltStateChangesEqual(test_ret, {})

    def test_mod_watch_test(self):
        '''
        Test service.mod_watch with test=True
        '''
        # Skip test if package was not installed
        if not PKG_INSTALLED:
            self.skipTest('Package containing service used by test was not installed')

        # Configure state parameters
        state_params = [
            'service.mod_watch',
            'name={0}'.format(self.service),
            'sfun=running',
            'test=True',
        ]

        # apply the state in test mode
        self.run_function('state.single', ['service.dead', self.service])
        test_ret = self.run_function('state.single', state_params)
        # Validate base state
        self.assertSaltNoneReturn(test_ret)
        self.assertInSaltComment('Service is set to be started', test_ret)
        self.assertSaltStateChangesEqual(test_ret, {})

    def test_mod_watch_no_sfun(self):
        '''
        Test service.mod_watch without sfun argument
        '''
        # Skip test if package was not installed
        if not PKG_INSTALLED:
            self.skipTest('Package containing service used by test was not installed')

        # Configure state parameters
        state_params = [
            'service.mod_watch',
            'name={0}'.format(self.service),
        ]

        # Apply the state
        mod_watch_ret = self.run_function('state.single', state_params)
        # Validate state return
        self.assertSaltFalseReturn(mod_watch_ret)
        self.assertInSaltComment('sfun must be set to either "running" or "dead"', mod_watch_ret)
        self.assertSaltStateChangesEqual(mod_watch_ret, {})

    def test_mod_watch_running(self):
        '''
        Test service.mod_watch with sfun=running
        '''
        # Skip test if package was not installed
        if not PKG_INSTALLED:
            self.skipTest('Package containing service used by test was not installed')

        # Configure state parameters
        state_params = [
            'service.mod_watch',
            'name={0}'.format(self.service),
            'sfun=running',
        ]

        # Apply state
        self.run_function('state.single', ['service.dead', self.service])
        running_ret = self.run_function('state.single', state_params)
        # Validate state return
        self.assertSaltTrueReturn(running_ret)
        self.assertInSaltComment('Service started', running_ret)
        self.assertSaltStateChangesEqual(running_ret, {self.service: True})

        # Apply state a second time
        already_running_ret = self.run_function('state.single', state_params)
        # Validate disabled state
        self.assertSaltTrueReturn(already_running_ret)
        self.assertInSaltComment('Service restarted', already_running_ret)
        self.assertSaltStateChangesEqual(already_running_ret, {self.service: True})

    def test_mod_watch_running_reload(self):
        '''
        Test service.mod_watch with sfun=running and reload=True
        '''
        # Skip test if package was not installed
        if not PKG_INSTALLED:
            self.skipTest('Package containing service used by test was not installed')

        # Skip test if functionality not present on system
        if not self.run_function('sys.doc', ['service.reload']):
            self.skipTest('service.reload function is not available on this system')

        # Configure state parameters
        state_params = [
            'service.mod_watch',
            'name={0}'.format(self.service),
            'sfun=running',
            'reload=True',
        ]

        # Apply state
        self.run_function('state.single', ['service.running', self.service, 'init_delay=3'])
        state_ret = self.run_function('state.single', state_params)
        # Validate disabled state
        self.assertSaltTrueReturn(state_ret)
        self.assertInSaltComment('Service reloaded', state_ret)
        self.assertSaltStateChangesEqual(state_ret, {self.service: True})

    def test_mod_watch_running_force(self):
        '''
        Test service.mod_watch with sfun=running and force=True
        '''
        # Skip test if package was not installed
        if not PKG_INSTALLED:
            self.skipTest('Package containing service used by test was not installed')

        # Skip test if functionality not present on system
        for func in 'reload', 'force_reload':
            if not self.run_function('sys.doc', ['service.{0}'.format(func)]):
                self.skipTest('service.{0} function is not available on this system'.format(func))

        # Configure state parameters
        state_params = [
            'service.mod_watch',
            'name={0}'.format(self.service),
            'sfun=running',
            'reload=True',
            'force=True',
        ]

        # Apply state
        self.run_function('state.single', ['service.running', self.service, 'init_delay=3'])
        state_ret = self.run_function('state.single', state_params)
        # Validate disabled state
        self.assertSaltTrueReturn(state_ret)
        self.assertInSaltComment('Service forcefully reloaded', state_ret)
        self.assertSaltStateChangesEqual(state_ret, {self.service: True})

    def test_mod_watch_running_full_restart(self):
        '''
        Test service.mod_watch with sfun=running and full_restart=True
        '''
        # Skip test if package was not installed
        if not PKG_INSTALLED:
            self.skipTest('Package containing service used by test was not installed')

        # Skip test if functionality not present on system
        if not self.run_function('sys.doc', ['service.full_restart']):
            self.skipTest('service.full_restart function is not available on this system')

        # Configure state parameters
        state_params = [
            'service.mod_watch',
            'name={0}'.format(self.service),
            'sfun=running',
            'full_restart=True',
        ]

        # Apply state
        self.run_function('state.single', ['service.running', self.service, 'init_delay=3'])
        state_ret = self.run_function('state.single', state_params)
        # Validate state return
        self.assertSaltTrueReturn(state_ret)
        self.assertInSaltComment('Service fully restarted', state_ret)
        self.assertSaltStateChangesEqual(state_ret, {self.service: True})

    def test_mod_watch_running_init_delay(self):
        '''
        Test service.mod_watch with sfun=running and init_delay
        '''
        # Skip test if package was not installed
        if not PKG_INSTALLED:
            self.skipTest('Package containing service used by test was not installed')

        # Configure state parameters
        init_delay = 6.2832
        state_params = [
            'service.mod_watch',
            'name={0}'.format(self.service),
            'sfun=running',
            'init_delay={0}'.format(init_delay),
        ]
        state_key = 'service_|-{0}_|-{0}_|-mod_watch'.format(self.service)

        # Apply state
        self.run_function('state.single', ['service.dead', self.service])
        state_ret = self.run_function('state.single', state_params)
        # Validate state return
        self.assertSaltTrueReturn(state_ret)
        self.assertInSaltComment('Service started', state_ret)
        self.assertSaltStateChangesEqual(state_ret, {self.service: True})
        self.assertTrue(state_ret[state_key]['duration']/1000 >= init_delay)

    def test_mod_watch_dead(self):
        '''
        Test service.mod_watch with sfun=dead
        '''
        # Skip test if package was not installed
        if not PKG_INSTALLED:
            self.skipTest('Package containing service used by test was not installed')

        # Configure state parameters
        state_params = [
            'service.mod_watch',
            'name={0}'.format(self.service),
            'sfun=dead',
        ]

        # Apply state
        self.run_function('state.single', ['service.running', self.service, 'init_delay=3'])
        mod_watch_ret = self.run_function('state.single', state_params)
        # Validate state return
        self.assertSaltTrueReturn(mod_watch_ret)
        self.assertInSaltComment('Service stopped', mod_watch_ret)
        self.assertSaltStateChangesEqual(mod_watch_ret, {self.service: True})

        # Apply state a second time
        already_mod_watch_ret = self.run_function('state.single', state_params)
        # Validate disabled state
        self.assertSaltTrueReturn(already_mod_watch_ret)
        self.assertInSaltComment('Service is already stopped', already_mod_watch_ret)
        self.assertSaltStateChangesEqual(already_mod_watch_ret, {})


if __name__ == '__main__':
    from integration import run_tests
    run_tests(ServiceTest)



# -*- coding: utf-8 -*-
'''
Display compact output data structure
=====================================

Example output::
'saltdev': {'test_|-always-passes_|-foo_|-succeed_without_changes': {'comment': 'Success!', 'name': 'foo', 'start_time': '05:16:26.111814', 'result': True, 'duration': 1, '__run_num__': 0, 'changes': {}}, 'test_|-my-custom-combo_|-foo_|-configurable_test_state': {'comment': 'bar.baz', 'name': 'foo', 'start_time': '05:16:26.117177', 'result': False, 'duration': 1, '__run_num__': 4, 'changes': {'testing': {'new': 'Something pretended to change', 'old': 'Unchanged'}}}, 'test_|-always-fails_|-foo_|-fail_without_changes': {'comment': 'Failure!', 'name': 'foo', 'start_time': '05:16:26.113124', 'result': False, 'duration': 1, '__run_num__': 1, 'changes': {}}, 'test_|-always-changes-and-succeeds_|-foo_|-succeed_with_changes': {'comment': 'Success!', 'name': 'foo', 'start_time': '05:16:26.114570', 'result': True, 'duration': 0, '__run_num__': 2, 'changes': {'testing': {'new': 'Something pretended to change', 'old': 'Unchanged'}}}, 'test_|-always-changes-and-fails_|-foo_|-fail_with_changes': {'comment': 'Failure!', 'name': 'foo', 'start_time': '05:16:26.115561', 'result': False, 'duration': 1, '__run_num__': 3, 'changes': {'testing': {'new': 'Something pretended to change', 'old': 'Unchanged'}}}}}{'myminion': {'foo': {'list': ['Hello', 'World'], 'bar': 'baz', 'dictionary': {'abc': 123, 'def': 456}}}}
'''
from __future__ import absolute_import

# Import salt libs
import salt.output.highstate


def output(data):
    '''
    Rather basic....
    '''
    tmp = {}
    for min_ in data:
        for process in data[min_]:
            add = False
            if data[min_][process]['result'] is False:
                add = True
            elif data[min_][process]['changes']:
                add = True
            if add is True:
                if min_ not in tmp:
                    tmp[min_] = {process: data[min_][process]}
                else:
                    tmp[min_][process] = {process: data[min_][process]}

    return salt.output.highstate.output(tmp)



# -*- coding: utf-8 -*-
'''
    :codeauthor: :email:`Rupesh Tare <rupesht@saltstack.com>`
'''

# Import Python libs
from __future__ import absolute_import

# Import Salt Testing Libs
from salttesting import TestCase, skipIf
from salttesting.mock import (
    MagicMock,
    patch,
    NO_MOCK,
    NO_MOCK_REASON
)

# Import Salt Libs
from salt.modules import img

# Globals
img.__salt__ = {}
img.__context__ = {}


@skipIf(NO_MOCK, NO_MOCK_REASON)
class ImgTestCase(TestCase):
    '''
    Test cases for salt.modules.img
    '''
    def test_mount_image(self):
        '''
        Test for Mount the named image and return the mount point
        '''
        mock = MagicMock(return_value='A')
        with patch.dict(img.__salt__, {'guestfs.mount': mock}):
            self.assertEqual(img.mount_image('location'), 'A')

        mock = MagicMock(side_effect=[{}, {'A': 'A', 'B': 'B'}])
        with patch.dict(img.__salt__, {'qemu_nbd.init': mock}):
            self.assertEqual(img.mount_image('location'), '')

            self.assertEqual(img.mount_image('location'), 'A')

        mock = MagicMock(return_value=None)
        with patch.dict(img.__salt__, mock):
            self.assertEqual(img.mount_image('location'), '')

    def test_umount_image(self):
        '''
        Test for Unmount an image mountpoint
        '''
        mock = MagicMock(return_value='A')
        with patch.dict(img.__salt__, {'qemu_nbd.clear': mock}):
            mock = MagicMock(return_value='B')
            with patch.dict(img.__context__, {'img.mnt_mnt': mock}):
                self.assertEqual(img.umount_image('mnt'), None)

    def test_bootstrap(self):
        '''
        Test for HIGHLY EXPERIMENTAL
        '''
        mock = MagicMock(return_value=None)
        with patch.dict(img.__salt__, {'img.make_image': mock}):
            self.assertEqual(img.bootstrap('location', 'size', 'fmt'), '')

if __name__ == '__main__':
    from integration import run_tests
    run_tests(ImgTestCase, needs_daemon=False)



# -*- coding: utf-8 -*-
'''
    :codeauthor: :email:`Rahul Handay <rahulha@saltstack.com>`
'''

# Import Python libs
from __future__ import absolute_import

# Import Salt Testing Libs
from salttesting import TestCase, skipIf
from salttesting.mock import (
    MagicMock,
    patch,
    NO_MOCK,
    NO_MOCK_REASON
)

# Import Salt Libs
from salt.modules import bluez
from salt.exceptions import CommandExecutionError
import salt.utils.validate.net


# Globals
bluez.__salt__ = {}


class MockBluetooth(object):
    '''
        Mock class for bluetooth
    '''
    def __init__(self):
        pass

    @staticmethod
    def discover_devices(lookup_names):
        '''
            Mock method to return all Discoverable devices
        '''
        return [['a', 'b', 'c'], ['d', 'e', 'f']]

bluez.bluetooth = MockBluetooth


@skipIf(NO_MOCK, NO_MOCK_REASON)
class BluezTestCase(TestCase):
    '''
        Test cases for salt.modules.bluez
    '''
    def test_version(self):
        '''
            Test if return bluetooth version
        '''
        mock = MagicMock(return_value="5.7")
        with patch.dict(bluez.__salt__, {'cmd.run': mock}):
            self.assertDictEqual(bluez.version(),
                                 {'PyBluez': '<= 0.18 (Unknown, but installed)',
                                  'Bluez': '5.7'})

    def test_address_(self):
        '''
            Test of getting address of bluetooth adapter
        '''
        mock = MagicMock(return_value='hci : hci0')
        with patch.dict(bluez.__salt__, {'cmd.run': mock}):
            self.assertDictEqual(bluez.address_(),
                                 {'hci ': {'device': 'hci ',
                                           'path': '/sys/class/bluetooth/hci '}})

    def test_power(self):
        '''
            Test of getting address of bluetooth adapter
        '''
        mock = MagicMock(return_value={})
        with patch.object(bluez, 'address_', mock):
            self.assertRaises(CommandExecutionError, bluez.power, "hci0", "on")

        mock = MagicMock(return_value={'hci0':
                                       {'device': 'hci0', 'power': 'on'}})
        with patch.object(bluez, 'address_', mock):
            mock = MagicMock(return_value="")
            with patch.dict(bluez.__salt__, {'cmd.run': mock}):
                self.assertTrue(bluez.power("hci0", "on"))

        mock = MagicMock(return_value={'hci0':
                                       {'device': 'hci0', 'power': 'on'}})
        with patch.object(bluez, 'address_', mock):
            mock = MagicMock(return_value="")
            with patch.dict(bluez.__salt__, {'cmd.run': mock}):
                self.assertFalse(bluez.power("hci0", "off"))

    def test_discoverable(self):
        '''
            Test of enabling bluetooth device
        '''
        mock = MagicMock(side_effect=[{}, {'hci0':
                                           {'device': 'hci0', 'power': 'on'}},
                                      {'hci0': {'device': 'hci0',
                                                'power': 'on'}}])
        with patch.object(bluez, 'address_', mock):
            self.assertRaises(CommandExecutionError,
                              bluez.discoverable, "hci0")

            mock = MagicMock(return_value="UP RUNNING ISCAN")
            with patch.dict(bluez.__salt__, {'cmd.run': mock}):
                self.assertTrue(bluez.discoverable("hci0"))

            mock = MagicMock(return_value="")
            with patch.dict(bluez.__salt__, {'cmd.run': mock}):
                self.assertFalse(bluez.discoverable("hci0"))

    def test_noscan(self):
        '''
            Test of turning off of scanning modes
        '''
        mock = MagicMock(side_effect=[{}, {'hci0':
                                           {'device': 'hci0', 'power': 'on'}},
                                      {'hci0': {'device': 'hci0',
                                                'power': 'on'}}])
        with patch.object(bluez, 'address_', mock):
            self.assertRaises(CommandExecutionError, bluez.noscan, "hci0")

            mock = MagicMock(return_value="SCAN")
            with patch.dict(bluez.__salt__, {'cmd.run': mock}):
                self.assertFalse(bluez.noscan("hci0"))

            mock = MagicMock(return_value="")
            with patch.dict(bluez.__salt__, {'cmd.run': mock}):
                self.assertTrue(bluez.noscan("hci0"))

    def test_scan(self):
        '''
            Test of scanning of bluetooth devices
        '''
        self.assertListEqual(bluez.scan(), [{'a': 'b'}, {'d': 'e'}])

    def test_block(self):
        '''
            Test of blocking specific bluetooth device
        '''
        mock = MagicMock(side_effect=[False, True])
        with patch.object(salt.utils.validate.net, 'mac', mock):
            self.assertRaises(CommandExecutionError,
                              bluez.block, "DE:AD:BE:EF:CA:ZE")

            mock = MagicMock(return_value="")
            with patch.dict(bluez.__salt__, {'cmd.run': mock}):
                self.assertIsNone(bluez.block("DE:AD:BE:EF:CA:FE"))

    def test_unblock(self):
        '''
            Test to unblock specific bluetooth device
        '''
        mock = MagicMock(side_effect=[False, True])
        with patch.object(salt.utils.validate.net, 'mac', mock):
            self.assertRaises(CommandExecutionError,
                              bluez.block, "DE:AD:BE:EF:CA:ZE")

            mock = MagicMock(return_value="")
            with patch.dict(bluez.__salt__, {'cmd.run': mock}):
                self.assertIsNone(bluez.unblock("DE:AD:BE:EF:CA:FE"))

    def test_pair(self):
        '''
            Test to pair bluetooth adapter with a device
        '''
        mock = MagicMock(side_effect=[False, True, True])
        with patch.object(salt.utils.validate.net, 'mac', mock):
            self.assertRaises(CommandExecutionError,
                              bluez.pair, "DE:AD:BE:EF:CA:FE", "1234")

            self.assertRaises(CommandExecutionError,
                              bluez.pair, "DE:AD:BE:EF:CA:FE", "abcd")

            mock = MagicMock(return_value={'device': 'hci0'})
            with patch.object(bluez, 'address_', mock):
                mock = MagicMock(return_value="Ok")
                with patch.dict(bluez.__salt__, {'cmd.run': mock}):
                    self.assertListEqual(bluez.
                                         pair("DE:AD:BE:EF:CA:FE", "1234"),
                                         ["Ok"])

    def test_unpair(self):
        '''
            Test to unpair bluetooth adaptor with a device
        '''
        mock = MagicMock(side_effect=[False, True])
        with patch.object(salt.utils.validate.net, 'mac', mock):
            self.assertRaises(CommandExecutionError,
                              bluez.unpair, "DE:AD:BE:EF:CA:FE")

            mock = MagicMock(return_value="Ok")
            with patch.dict(bluez.__salt__, {'cmd.run': mock}):
                self.assertListEqual(bluez.unpair("DE:AD:BE:EF:CA:FE"), ["Ok"])

    def test_start(self):
        '''
            Test to start bluetooth service
        '''
        mock = MagicMock(return_value="Ok")
        with patch.dict(bluez.__salt__, {'service.start': mock}):
            self.assertEqual(bluez.start(), "Ok")

    def test_stop(self):
        '''
            Test to stop bluetooth service
        '''
        mock = MagicMock(return_value="Ok")
        with patch.dict(bluez.__salt__, {'service.stop': mock}):
            self.assertEqual(bluez.stop(), "Ok")

if __name__ == '__main__':
    from integration import run_tests
    run_tests(BluezTestCase, needs_daemon=False)



# -*- coding: utf-8 -*-
'''

Manage PagerDuty escalation policies.

Schedules and users can be referenced by pagerduty ID, or by name, or by email address.

For example:

.. code-block:: yaml

    ensure test escalation policy:
        pagerduty_escalation_policy.present:
            - name: bruce test escalation policy
            - escalation_rules:
                - targets:
                    - type: schedule
                      id: 'bruce test schedule level1'
                    - type: user
                      id: 'Bruce Sherrod'
                  escalation_delay_in_minutes: 15
                - targets:
                    - type: schedule
                      id: 'bruce test schedule level2'
                  escalation_delay_in_minutes: 15
                - targets:
                    - type: user
                      id: 'Bruce TestUser1'
                    - type: user
                      id: 'Bruce TestUser2'
                    - type: user
                      id: 'Bruce TestUser3'
                    - type: user
                      id:  'bruce+test4@lyft.com'
                  escalation_delay_in_minutes: 15
'''


def __virtual__():
    '''
    Only load if the pygerduty module is available in __salt__
    '''
    return 'pagerduty_escalation_policy' if 'pagerduty_util.get_resource' in __salt__ else False


def present(profile='pagerduty', subdomain=None, api_key=None, **kwargs):
    '''
    Ensure that a pagerduty escalation policy exists.  Will create or update as needed.

    This method accepts as args everything defined in
    https://developer.pagerduty.com/documentation/rest/escalation_policies/create.
    In addition, user and schedule id's will be translated from name (or email address)
    into PagerDuty unique ids.  For example:

    .. code-block:: yaml

    pagerduty_escalation_policy.present:
        - name: bruce test escalation policy
        - escalation_rules:
            - targets:
                - type: schedule
                  id: 'bruce test schedule level1'
                - type: user
                  id: 'Bruce Sherrod'

    In this example, 'Bruce Sherrod' will be looked up and replaced with the
    PagerDuty id (usually a 7 digit all-caps string, e.g. PX6GQL7)

    '''
    # for convenience, we accept id, name, or email for users
    # and we accept the id or name for schedules
    for escalation_rule in kwargs['escalation_rules']:
        for target in escalation_rule['targets']:
            target_id = None
            if target['type'] == 'user':
                user = __salt__['pagerduty_util.get_resource']('users',
                                                               target['id'],
                                                               ['email', 'name', 'id'],
                                                               profile=profile,
                                                               subdomain=subdomain,
                                                               api_key=api_key)
                if user:
                    target_id = user['id']
            elif target['type'] == 'schedule':
                schedule = __salt__['pagerduty_util.get_resource']('schedules',
                                                                   target['id'],
                                                                   ['name', 'id'],
                                                                   profile=profile,
                                                                   subdomain=subdomain,
                                                                   api_key=api_key)
                if schedule:
                    target_id = schedule['schedule']['id']
            if target_id is None:
                raise Exception('unidentified target: {0}'.format(str(target)))
            target['id'] = target_id

    r = __salt__['pagerduty_util.resource_present']('escalation_policies',
                                                    ['name', 'id'],
                                                    _diff,
                                                    profile,
                                                    subdomain,
                                                    api_key,
                                                    **kwargs)
    return r


def absent(profile='pagerduty', subdomain=None, api_key=None, **kwargs):
    '''
    Ensure that a PagerDuty escalation policy does not exist.
    Accepts all the arguments that pagerduty_escalation_policy.present accepts;
    but ignores all arguments except the name.

    Name can be the escalation policy id or the escalation policy name.
    '''
    r = __salt__['pagerduty_util.resource_absent']('escalation_policies',
                                                   ['name', 'id'],
                                                   profile,
                                                   subdomain,
                                                   api_key,
                                                   **kwargs)
    return r


def _diff(state_data, resource_object):
    '''helper method to compare salt state info with the PagerDuty API json structure,
    and determine if we need to update.

    returns the dict to pass to the PD API to perform the update, or empty dict if no update.
    '''
    objects_differ = None

    for k, v in state_data.items():
        if k == 'escalation_rules':
            v = _escalation_rules_to_string(v)
            resource_value = _escalation_rules_to_string(resource_object[k])
        else:
            resource_value = resource_object[k]
        if v != resource_value:
            objects_differ = '{0} {1} {2}'.format(k, v, resource_value)
            break

    if objects_differ:
        return state_data
    else:
        return {}


def _escalation_rules_to_string(escalation_rules):
    'convert escalation_rules dict to a string for comparison'
    result = ''
    for rule in escalation_rules:
        result += 'escalation_delay_in_minutes: {0} '.format(rule['escalation_delay_in_minutes'])
        for target in rule['targets']:
            result += '{0}:{1} '.format(target['type'], target['id'])
    return result



# -*- coding: utf-8 -*-
'''
Manage X509 Certificates

.. versionadded:: 2015.8.0

This module can enable managing a complete PKI infrastructure including creating private keys, CA's,
certificates and CRLs. It includes the ability to generate a private key on a server, and have the
corresponding public key sent to a remote CA to create a CA signed certificate. This can be done in
a secure manner, where private keys are always generated locally and never moved across the network.

Here is a simple example scenario. In this example ``ca`` is the ca server,
and ``www`` is a web server that needs a certificate signed by ``ca``.

For remote signing, peers must be permitted to remotely call the
:mod:`sign_remote_certificate <salt.modules.x509.sign_remote_certificate>` function.


/etc/salt/master.d/peer.conf

.. code-block:: yaml

    peer:
      .*:
        - x509.sign_remote_certificate


/srv/salt/top.sls

.. code-block:: yaml

    base:
      '*':
        - cert
      'ca':
        - ca
      'www':
        - www


This state creates the CA key, certificate and signing policy. It also publishes the certificate to
the mine where it can be easily retrieved by other minions.

/srv/salt/ca.sls

.. code-block:: yaml

    salt-minion:
      service.running:
        - enable: True
        - listen:
          - file: /etc/salt/minion.d/signing_policies.conf

    /etc/salt/minion.d/signing_policies.conf:
      file.managed:
        - source: salt://signing_policies.conf

    /etc/pki:
      file.directory: []

    /etc/pki/issued_certs:
      file.directory: []

    /etc/pki/ca.key:
      x509.private_key_managed:
        - bits: 4096
        - backup: True
        - require:
          - file: /etc/pki

    /etc/pki/ca.crt:
      x509.certificate_managed:
        - signing_private_key: /etc/pki/ca.key
        - CN: ca.example.com
        - C: US
        - ST: Utah
        - L: Salt Lake City
        - basicConstraints: "critical CA:true"
        - keyUsage: "critical cRLSign, keyCertSign"
        - subjectKeyIdentifier: hash
        - authorityKeyIdentifier: keyid,issuer:always
        - days_valid: 3650
        - days_remaining: 0
        - backup: True
        - require:
          - x509: /etc/pki/ca.key

    mine.send:
      module.run:
        - func: x509.get_pem_entries
        - kwargs:
            glob_path: /etc/pki/ca.crt
        - onchanges:
          - x509: /etc/pki/ca.crt


The signing policy defines properties that override any property requested or included in a CRL. It also
can define a restricted list of minons which are allowed to remotely invoke this signing policy.

/srv/salt/signing_policies.conf

.. code-block:: yaml

    x509_signing_policies:
      www:
        - minions: 'www'
        - signing_private_key: /etc/pki/ca.key
        - signing_cert: /etc/pki/ca.crt
        - C: US
        - ST: Utah
        - L: Salt Lake City
        - basicConstraints: "critical CA:false"
        - keyUsage: "critical cRLSign, keyCertSign"
        - subjectKeyIdentifier: hash
        - authorityKeyIdentifier: keyid,issuer:always
        - days_valid: 90
        - copypath: /etc/pki/issued_certs/


This state will instruct all minions to trust certificates signed by our new CA.
Using jinja to strip newlines from the text avoids dealing with newlines in the rendered yaml,
and the  :mod:`sign_remote_certificate <salt.states.x509.sign_remote_certificate>` state will
handle properly formatting the text before writing the output.

/srv/salt/cert.sls

.. code-block:: yaml

    /usr/local/share/ca-certificates:
      file.directory: []

    /usr/local/share/ca-certificates/intca.crt:
      x509.pem_managed:
        - text: {{ salt['mine.get']('ca', 'x509.get_pem_entries')['ca']['/etc/pki/ca.crt']|replace('\\n', '') }}


This state creates a private key then requests a certificate signed by ca according to the www policy.

/srv/salt/www.sls

.. code-block:: yaml

    /etc/pki/www.key:
      x509.private_key_managed:
        - bits: 4096

    /etc/pki/www.crt:
      x509.certificate_managed:
        - ca_server: ca
        - signing_policy: www
        - public_key: /etc/pki/www.key
        - CN: www.example.com
        - days_remaining: 30
        - backup: True

'''

# Import Python Libs
from __future__ import absolute_import
import datetime
import os
import re
import copy

# Import Salt Libs
import salt.exceptions
import salt.utils

# Import 3rd-party libs
import salt.ext.six as six


def __virtual__():
    '''
    only load this module if the corresponding execution module is loaded
    '''
    if 'x509.get_pem_entry' in __salt__:
        return 'x509'
    else:
        return (False, 'Could not load x509 state: m2crypto unavailable')


def _revoked_to_list(revs):
    '''
    Turn the mess of OrderedDicts and Lists into a list of dicts for
    use in the CRL module.
    '''
    list_ = []

    for rev in revs:
        for rev_name, props in six.iteritems(rev):             # pylint: disable=unused-variable
            dict_ = {}
            for prop in props:
                for propname, val in six.iteritems(prop):
                    if isinstance(val, datetime.datetime):
                        val = val.strftime('%Y-%m-%d %H:%M:%S')
                    dict_[propname] = val
            list_.append(dict_)

    return list_


def private_key_managed(name,
                        bits=2048,
                        new=False,
                        backup=False):
    '''
    Manage a private key's existence.

    name:
        Path to the private key

    bits:
        Key length in bits. Default 2048.

    new:
        Always create a new key. Defaults to False.
        Combining new with :mod:`prereq <salt.states.requsities.preqreq>` can allow key rotation
        whenever a new certificiate is generated.

    backup:
        When replacing an existing file, backup the old file onthe minion.
        Default is False.

    Example:

    The jinja templating in this example ensures a private key is generated if the file doesn't exist
    and that a new private key is generated whenever the certificate that uses it is to be renewed.

    .. code-block:: yaml

        /etc/pki/www.key:
          x509.private_key_managed:
            - bits: 4096
            - new: True
            {% if salt['file.file_exists']('/etc/pki/ca.key') -%}
            - prereq:
              - x509: /etc/pki/www.crt
            {%- endif %}
    '''
    ret = {'name': name, 'changes': {}, 'result': False, 'comment': ''}

    current_bits = 0
    if os.path.isfile(name):
        try:
            current_bits = __salt__['x509.get_private_key_size'](private_key=name)
            current = "{0} bit private key".format(current_bits)
        except salt.exceptions.SaltInvocationError:
            current = '{0} is not a valid Private Key.'.format(name)
    else:
        current = '{0} does not exist.'.format(name)

    if current_bits == bits and not new:
        ret['result'] = True
        ret['comment'] = 'The Private key is already in the correct state'
        return ret

    ret['changes'] = {
            'old': current,
            'new': "{0} bit private key".format(bits)}

    if __opts__['test'] is True:
        ret['result'] = None
        ret['comment'] = 'The Private Key "{0}" will be updated.'.format(name)
        return ret

    if os.path.isfile(name) and backup:
        bkroot = os.path.join(__opts__['cachedir'], 'file_backup')
        salt.utils.backup_minion(name, bkroot)

    ret['comment'] = __salt__['x509.create_private_key'](path=name, bits=bits)
    ret['result'] = True

    return ret


def csr_managed(name,
                backup=False,
                **kwargs):
    '''
    Manage a Certificate Signing Request

    name:
        Path to the CSR

    properties:
        The properties to be added to the certificate request, including items like subject, extensions
        and public key. See above for valid properties.

    Example:

    .. code-block:: yaml

        /etc/pki/mycert.csr:
          x509.csr_managed:
             - public_key: /etc/pki/mycert.key
             - CN: www.example.com
             - C: US
             - ST: Utah
             - L: Salt Lake City
             - keyUsage: 'critical dataEncipherment'
    '''
    ret = {'name': name, 'changes': {}, 'result': False, 'comment': ''}

    if os.path.isfile(name):
        try:
            current = __salt__['x509.read_csr'](csr=name)
        except salt.exceptions.SaltInvocationError:
            current = '{0} is not a valid CSR.'.format(name)
    else:
        current = '{0} does not exist.'.format(name)

    new_csr = __salt__['x509.create_csr'](text=True, **kwargs)
    new = __salt__['x509.read_csr'](csr=new_csr)

    if current == new:
        ret['result'] = True
        ret['comment'] = 'The CSR is already in the correct state'
        return ret

    ret['changes'] = {
            'old': current,
            'new': new, }

    if __opts__['test'] is True:
        ret['result'] = None
        ret['comment'] = 'The CSR {0} will be updated.'.format(name)

    if os.path.isfile(name) and backup:
        bkroot = os.path.join(__opts__['cachedir'], 'file_backup')
        salt.utils.backup_minion(name, bkroot)

    ret['comment'] = __salt__['x509.write_pem'](text=new_csr, path=name, pem_type="CERTIFICATE REQUEST")
    ret['result'] = True

    return ret


def certificate_managed(name,
                        days_remaining=90,
                        backup=False,
                        **kwargs):
    '''
    Manage a Certificate

    name:
        Path to the certificate

    days_remaining:
        The minimum number of days remaining when the certificate should be recreated. Default is 90. A
        value of 0 disables automatic renewal.

    backup:
        When replacing an existing file, backup the old file on the minion. Default is False.

    kwargs:
        Any arguments supported by :mod:`x509.create_certificate <salt.modules.x509.create_certificate>`
        are supported.

    Examples:

    .. code-block:: yaml

        /etc/pki/ca.crt:
          x509.certificate_managed:
            - signing_private_key: /etc/pki/ca.key
            - CN: ca.example.com
            - C: US
            - ST: Utah
            - L: Salt Lake City
            - basicConstraints: "critical CA:true"
            - keyUsage: "critical cRLSign, keyCertSign"
            - subjectKeyIdentifier: hash
            - authorityKeyIdentifier: keyid,issuer:always
            - days_valid: 3650
            - days_remaining: 0
            - backup: True


    .. code-block:: yaml

        /etc/ssl/www.crt:
          x509.certificate_managed:
            - ca_server: pki
            - signing_policy: www
            - public_key: /etc/ssl/www.key
            - CN: www.example.com
            - days_valid: 90
            - days_remaining: 30
            - backup: True

    '''
    ret = {'name': name, 'changes': {}, 'result': False, 'comment': ''}

    current_days_remaining = 0
    current_comp = {}

    if os.path.isfile(name):
        try:
            current = __salt__['x509.read_certificate'](certificate=name)
            current_comp = copy.deepcopy(current)
            if 'serial_number' not in kwargs:
                current_comp.pop('Serial Number')
                if 'signing_cert' not in kwargs:
                    try:
                        current_comp['X509v3 Extensions']['authorityKeyIdentifier'] = (
                            re.sub(r'serial:([0-9A-F]{2}:)*[0-9A-F]{2}', 'serial:--',
                                current_comp['X509v3 Extensions']['authorityKeyIdentifier']))
                    except KeyError:
                        pass
            current_comp.pop('Not Before')
            current_comp.pop('MD5 Finger Print')
            current_comp.pop('SHA1 Finger Print')
            current_comp.pop('SHA-256 Finger Print')
            current_notafter = current_comp.pop('Not After')
            current_days_remaining = (
                    datetime.datetime.strptime(current_notafter, '%Y-%m-%d %H:%M:%S') -
                    datetime.datetime.now()).days
            if days_remaining == 0:
                days_remaining = current_days_remaining - 1
        except salt.exceptions.SaltInvocationError:
            current = '{0} is not a valid Certificate.'.format(name)
    else:
        current = '{0} does not exist.'.format(name)

    if 'ca_server' in kwargs and 'signing_policy' not in kwargs:
        raise salt.exceptions.SaltInvocationError('signing_policy must be specified if ca_server is.')

    new = __salt__['x509.create_certificate'](testrun=True, **kwargs)

    if isinstance(new, dict):
        new_comp = copy.deepcopy(new)
        new.pop('Issuer Public Key')
        if 'serial_number' not in kwargs:
            new_comp.pop('Serial Number')
            if 'signing_cert' not in kwargs:
                try:
                    new_comp['X509v3 Extensions']['authorityKeyIdentifier'] = (
                        re.sub(r'serial:([0-9A-F]{2}:)*[0-9A-F]{2}', 'serial:--',
                            new_comp['X509v3 Extensions']['authorityKeyIdentifier']))
                except KeyError:
                    pass
        new_comp.pop('Not Before')
        new_comp.pop('Not After')
        new_comp.pop('MD5 Finger Print')
        new_comp.pop('SHA1 Finger Print')
        new_comp.pop('SHA-256 Finger Print')
        new_issuer_public_key = new_comp.pop('Issuer Public Key')
    else:
        new_comp = new

    if (current_comp == new_comp and
            current_days_remaining > days_remaining and
            __salt__['x509.verify_signature'](name, new_issuer_public_key)):
        ret['result'] = True
        ret['comment'] = 'The certificate is already in the correct state'
        return ret

    ret['changes'] = {
            'old': current,
            'new': new, }

    if __opts__['test'] is True:
        ret['result'] = None
        ret['comment'] = 'The certificate {0} will be updated.'.format(name)
        return ret

    if os.path.isfile(name) and backup:
        bkroot = os.path.join(__opts__['cachedir'], 'file_backup')
        salt.utils.backup_minion(name, bkroot)

    ret['comment'] = __salt__['x509.create_certificate'](path=name, **kwargs)
    ret['result'] = True

    return ret


def crl_managed(name,
                signing_private_key,
                signing_cert=None,
                revoked=None,
                days_valid=100,
                days_remaining=30,
                include_expired=False,
                backup=False,):
    '''
    Manage a Certificate Revocation List

    name:
        Path to the certificate

    signing_private_key:
        The private key that will be used to sign this crl. This is
        usually your CA's private key.

    signing_cert:
        The certificate of the authority that will be used to sign this crl.
        This is usually your CA's certificate.

    revoked:
        A list of certificates to revoke. Must include either a serial number or a
        the certificate itself. Can optionally include the revocation date and
        notAfter date from the certificate. See example below for details.

    days_valid:
        The number of days the certificate should be valid for. Default is 100.

    days_remaining:
        The crl should be automatically recreated if there are less than ``days_remaining``
        days until the crl expires. Set to 0 to disable automatic renewal. Default is 30.

    include_expired:
        Include expired certificates in the CRL. Default is ``False``.

    backup:
        When replacing an existing file, backup the old file onthe minion. Default is False.

    Example:

    .. code-block:: yaml

        /etc/pki/ca.crl:
          x509.crl_managed:
            - signing_private_key: /etc/pki/myca.key
            - signing_cert: /etc/pki/myca.crt
            - revoked:
              - compromized_Web_key:
                - certificate: /etc/pki/certs/badweb.crt
                - revocation_date: 2015-03-01 00:00:00
                - reason: keyCompromise
              - terminated_vpn_user:
                - serial_number: D6:D2:DC:D8:4D:5C:C0:F4
                - not_after: 2016-01-01 00:00:00
                - revocation_date: 2015-02-25 00:00:00
                - reason: cessationOfOperation
    '''
    ret = {'name': name, 'changes': {}, 'result': False, 'comment': ''}

    if revoked is None:
        revoked = []

    revoked = _revoked_to_list(revoked)

    current_days_remaining = 0
    current_comp = {}

    if os.path.isfile(name):
        try:
            current = __salt__['x509.read_crl'](crl=name)
            current_comp = current.copy()
            current_comp.pop('Last Update')
            current_notafter = current_comp.pop('Next Update')
            current_days_remaining = (
                    datetime.datetime.strptime(current_notafter, '%Y-%m-%d %H:%M:%S') -
                    datetime.datetime.now()).days
            if days_remaining == 0:
                days_remaining = current_days_remaining - 1
        except salt.exceptions.SaltInvocationError:
            current = '{0} is not a valid CRL.'.format(name)
    else:
        current = '{0} does not exist.'.format(name)

    new_crl = __salt__['x509.create_crl'](text=True, signing_private_key=signing_private_key,
            signing_cert=signing_cert, revoked=revoked, days_valid=days_valid, include_expired=include_expired)

    new = __salt__['x509.read_crl'](crl=new_crl)
    new_comp = new.copy()
    new_comp.pop('Last Update')
    new_comp.pop('Next Update')

    if (current_comp == new_comp and
            current_days_remaining > days_remaining and
            __salt__['x509.verify_crl'](name, signing_cert)):

        ret['result'] = True
        ret['comment'] = 'The crl is already in the correct state'
        return ret

    ret['changes'] = {
            'old': current,
            'new': new, }

    if __opts__['test'] is True:
        ret['result'] = None
        ret['comment'] = 'The crl {0} will be updated.'.format(name)
        return ret

    if os.path.isfile(name) and backup:
        bkroot = os.path.join(__opts__['cachedir'], 'file_backup')
        salt.utils.backup_minion(name, bkroot)

    ret['comment'] = __salt__['x509.write_pem'](text=new_crl, path=name, pem_type='X509 CRL')
    ret['result'] = True

    return ret


def pem_managed(name,
                text,
                backup=False):
    '''
    Manage the contents of a PEM file directly with the content in text, ensuring correct formatting.

    name:
        The path to the file to manage

    text:
        The PEM formatted text to write.

    backup:
        When replacing an existing file, backup the old file on the minion. Default is False.
    '''
    ret = {'name': name, 'changes': {}, 'result': False, 'comment': ''}

    new = __salt__['x509.get_pem_entry'](text=text)

    try:
        with salt.utils.fopen(name) as fp_:
            current = fp_.read()
    except (OSError, IOError):
        current = '{0} does not exist or is unreadable'.format(name)

    if new == current:
        ret['result'] = True
        ret['comment'] = 'The file is already in the correct state'
        return ret

    ret['changes']['new'] = new
    ret['changes']['old'] = current

    if __opts__['test'] is True:
        ret['result'] = None
        ret['comment'] = 'The file {0} will be updated.'.format(name)
        return ret

    if os.path.isfile(name) and backup:
        bkroot = os.path.join(__opts__['cachedir'], 'file_backup')
        salt.utils.backup_minion(name, bkroot)

    ret['comment'] = __salt__['x509.write_pem'](text=text, path=name)
    ret['result'] = True

    return ret



# -*- coding: utf-8 -*-
'''
Module for running imgadm command on SmartOS
'''
from __future__ import absolute_import

# Import Python libs
import logging
import json

# Import Salt libs
import salt.utils
import salt.utils.decorators as decorators

log = logging.getLogger(__name__)

# Function aliases
__func_alias__ = {
    'list_installed': 'list',
    'update_installed': 'update',
    'import_image': 'import'
}

# Define the module's virtual name
__virtualname__ = 'imgadm'


@decorators.memoize
def _check_imgadm():
    '''
    Looks to see if imgadm is present on the system
    '''
    return salt.utils.which('imgadm')


def _exit_status(retcode):
    '''
    Translate exit status of imgadm
    '''
    ret = {0: 'Successful completion.',
           1: 'An error occurred.',
           2: 'Usage error.',
           3: 'Image not installed.'}[retcode]
    return ret


def _parse_image_meta(image=None, detail=False):
    if not image:
        return {}

    if detail:
        return {
            'name': image['manifest']['name'],
            'version': image['manifest']['version'],
            'os': image['manifest']['os'],
            'description': image['manifest']['description'],
            'published': image['manifest']['published_at'],
            'source': image['source']
        }
    else:
        return '{name}@{version} [{date}]'.format(
                name=image['manifest']['name'],
                version=image['manifest']['version'],
                date=image['manifest']['published_at'],
            )


def __virtual__():
    '''
    Provides imgadm only on SmartOS
    '''
    if salt.utils.is_smartos_globalzone() and _check_imgadm():
        return __virtualname__
    return False


def version():
    '''
    Return imgadm version

    CLI Example:

    .. code-block:: bash

        salt '*' imgadm.version
    '''
    ret = {}
    imgadm = _check_imgadm()
    cmd = '{0} --version'.format(imgadm)
    res = __salt__['cmd.run'](cmd).splitlines()
    ret = res[0].split()
    return ret[-1]


def update_installed(uuid=''):
    '''
    Gather info on unknown image(s) (locally installed)

    uuid : string
        Specifies uuid of image

    CLI Example:

    .. code-block:: bash

        salt '*' imgadm.update [uuid]
    '''
    imgadm = _check_imgadm()
    if imgadm:
        cmd = '{0} update {1}'.format(imgadm, uuid).rstrip()
        __salt__['cmd.run'](cmd)
    return {}


def avail(search=None, verbose=False):
    '''
    Return a list of available images

    search : string
        Specifies search keyword
    verbose : boolean (False)
        Specifies verbose output

    CLI Example:

    .. code-block:: bash

        salt '*' imgadm.avail [percona]
        salt '*' imgadm.avail verbose=True
    '''
    ret = {}
    imgadm = _check_imgadm()
    cmd = '{0} avail -j'.format(imgadm)
    res = __salt__['cmd.run_all'](cmd)
    retcode = res['retcode']
    result = {}
    if retcode != 0:
        ret['Error'] = _exit_status(retcode)
        return ret

    for image in json.loads(res['stdout']):
        if image['manifest']['disabled'] or not image['manifest']['public']:
            continue
        if search and search not in image['manifest']['name']:
            # we skip if we are searching but don't have a match
            continue
        result[image['manifest']['uuid']] = _parse_image_meta(image, verbose)

    return result


def list_installed(verbose=False):
    '''
    Return a list of installed images

    verbose : boolean (False)
        Specifies verbose output

    CLI Example:

    .. code-block:: bash

        salt '*' imgadm.list [verbose=True]
    '''
    ret = {}
    imgadm = _check_imgadm()
    cmd = '{0} list -j'.format(imgadm)
    res = __salt__['cmd.run_all'](cmd)
    retcode = res['retcode']
    result = {}
    if retcode != 0:
        ret['Error'] = _exit_status(retcode)
        return ret

    for image in json.loads(res['stdout']):
        result[image['manifest']['uuid']] = _parse_image_meta(image, verbose)

    return result


def show(uuid=None):
    '''
    Show manifest of a given image

    CLI Example:

    .. code-block:: bash

        salt '*' imgadm.show e42f8c84-bbea-11e2-b920-078fab2aab1f
    '''
    ret = {}
    if not uuid:
        ret['Error'] = 'UUID parameter is mandatory'
        return ret
    imgadm = _check_imgadm()
    cmd = '{0} show {1}'.format(imgadm, uuid)
    res = __salt__['cmd.run_all'](cmd, python_shell=False)
    retcode = res['retcode']
    if retcode != 0:
        ret['Error'] = _exit_status(retcode)
        return ret
    ret = json.loads(res['stdout'])
    return ret


def get(uuid=None):
    '''
    Return info on an installed image

    CLI Example:

    .. code-block:: bash

        salt '*' imgadm.get e42f8c84-bbea-11e2-b920-078fab2aab1f
    '''
    ret = {}
    if not uuid:
        ret['Error'] = 'UUID parameter is mandatory'
        return ret
    imgadm = _check_imgadm()
    cmd = '{0} get {1}'.format(imgadm, uuid)
    res = __salt__['cmd.run_all'](cmd, python_shell=False)
    retcode = res['retcode']
    if retcode != 0:
        ret['Error'] = _exit_status(retcode)
        return ret
    ret = json.loads(res['stdout'])
    return ret


def import_image(uuid=None, verbose=False):
    '''
    Import an image from the repository

    uuid : string
        Specifies uuid to import
    verbose : boolean (False)
        Specifies verbose output

    CLI Example:

    .. code-block:: bash

        salt '*' imgadm.import e42f8c84-bbea-11e2-b920-078fab2aab1f [verbose=True]
    '''
    ret = {}
    if not uuid:
        ret['Error'] = 'UUID parameter is mandatory'
        return ret
    imgadm = _check_imgadm()
    cmd = '{0} import {1}'.format(imgadm, uuid)
    res = __salt__['cmd.run_all'](cmd, python_shell=False)
    retcode = res['retcode']
    if retcode != 0:
        ret['Error'] = _exit_status(retcode)
        return ret

    return {uuid: _parse_image_meta(get(uuid), verbose)}


def delete(uuid=None):
    '''
    Remove an installed image

    uuid : string
        Specifies uuid to import

    CLI Example:

    .. code-block:: bash

        salt '*' imgadm.delete e42f8c84-bbea-11e2-b920-078fab2aab1f
    '''
    ret = {}
    if not uuid:
        ret['Error'] = 'UUID parameter is mandatory'
        return ret
    imgadm = _check_imgadm()
    cmd = '{0} delete {1}'.format(imgadm, uuid)
    res = __salt__['cmd.run_all'](cmd, python_shell=False)
    retcode = res['retcode']
    if retcode != 0:
        ret['Error'] = _exit_status(retcode)
        return ret
    # output: Deleted image d5b3865c-0804-11e5-be21-dbc4ce844ddc
    result = []
    for image in res['stdout'].splitlines():
        image = [var for var in image.split(" ") if var]
        result.append(image[2])

    return result


def vacuum(verbose=False):
    '''
    Remove unused images

    verbose : boolean (False)
        Specifies verbose output

    CLI Example:

    .. code-block:: bash

        salt '*' imgadm.vacuum [verbose=True]
    '''
    ret = {}
    imgadm = _check_imgadm()
    cmd = '{0} vacuum -f'.format(imgadm)
    res = __salt__['cmd.run_all'](cmd)
    retcode = res['retcode']
    if retcode != 0:
        ret['Error'] = _exit_status(retcode)
        return ret
    # output: Deleted image d5b3865c-0804-11e5-be21-dbc4ce844ddc (lx-centos-6@20150601)
    result = {}
    for image in res['stdout'].splitlines():
        image = [var for var in image.split(" ") if var]
        result[image[2]] = {
            'name': image[3][1:image[3].index('@')],
            'version': image[3][image[3].index('@')+1:-1]
        }
    if verbose:
        return result
    else:
        return list(result.keys())


# vim: tabstop=4 expandtab shiftwidth=4 softtabstop=4



# -*- coding: utf-8 -*-

# Import python libs
from __future__ import absolute_import

# Import salt testing libs
from salttesting.unit import skipIf, TestCase
from salttesting.mock import NO_MOCK, NO_MOCK_REASON, MagicMock
from salttesting.helpers import ensure_in_syspath
ensure_in_syspath('../../')

# Import salt libs
from salt.utils.odict import OrderedDict
from salt.modules import jboss7

try:
    # will pass if executed along with other tests
    __salt__
except NameError:
    # if executed separately we need to export __salt__ dictionary ourselves
    from salt.ext.six.moves import builtins as __builtin__
    __builtin__.__salt__ = {}


@skipIf(NO_MOCK, NO_MOCK_REASON)
class JBoss7TestCase(TestCase):
    jboss_config = {}
    org_run_operation = None

    def setUp(self):
        if 'jboss7_cli.run_operation' in __salt__:
            self.org_run_operation = __salt__['jboss7_cli.run_operation']
        __salt__['jboss7_cli.run_operation'] = MagicMock()

    def tearDown(self):
        if self.org_run_operation is not None:
            __salt__['jboss7_cli.run_operation'] = self.org_run_operation

    def test_create_simple_binding(self):
        jboss7.create_simple_binding(self.jboss_config, 'java:global/env', 'DEV')

        __salt__['jboss7_cli.run_operation'].assert_called_with(self.jboss_config, '/subsystem=naming/binding="java:global/env":add(binding-type=simple, value="DEV")')

    def test_create_simple_binding_with_backslash(self):
        jboss7.create_simple_binding(self.jboss_config, 'java:global/env', r'DEV\2')

        __salt__['jboss7_cli.run_operation'].assert_called_with(self.jboss_config, r'/subsystem=naming/binding="java:global/env":add(binding-type=simple, value="DEV\\\\2")')

    def test_update_binding(self):
        jboss7.update_simple_binding(self.jboss_config, 'java:global/env', 'INT')

        __salt__['jboss7_cli.run_operation'].assert_called_with(self.jboss_config, '/subsystem=naming/binding="java:global/env":write-attribute(name=value, value="INT")')

    def test_update_binding_with_backslash(self):
        jboss7.update_simple_binding(self.jboss_config, 'java:global/env', r'INT\2')

        __salt__['jboss7_cli.run_operation'].assert_called_with(self.jboss_config, r'/subsystem=naming/binding="java:global/env":write-attribute(name=value, value="INT\\\\2")')

    def test_read_binding(self):
        def cli_command_response(jboss_config, cli_command):
            if cli_command == '/subsystem=naming/binding="java:global/env":read-resource':
                return {'outcome': 'success',
                         'result': {
                             'binding-type': 'simple',
                             'value': 'DEV'
                        }
                }

        __salt__['jboss7_cli.run_operation'].side_effect = cli_command_response

        result = jboss7.read_simple_binding(self.jboss_config, 'java:global/env')
        self.assertEqual(result['outcome'], 'success')
        self.assertEqual(result['result']['value'], 'DEV')

    def test_create_datasource_all_properties_included(self):
        def cli_command_response(jboss_config, cli_command, fail_on_error=False):
            if cli_command == '/subsystem=datasources/data-source="appDS":read-resource-description':
                return {'outcome': 'success',
                        'result': {
                            'attributes': {
                                'driver-name': {'type': 'STRING'},
                                'connection-url': {'type': 'STRING'},
                                'jndi-name': {'type': 'STRING'},
                                'user-name': {'type': 'STRING'},
                                'password': {'type': 'STRING'}
                            }
                        }
                }

        __salt__['jboss7_cli.run_operation'].side_effect = cli_command_response

        datasource_properties = OrderedDict()
        datasource_properties['driver-name'] = 'mysql'
        datasource_properties['connection-url'] = 'jdbc:mysql://localhost:3306/app'
        datasource_properties['jndi-name'] = 'java:jboss/datasources/appDS'
        datasource_properties['user-name'] = 'app'
        datasource_properties['password'] = 'app_password'

        jboss7.create_datasource(self.jboss_config, 'appDS', datasource_properties)

        __salt__['jboss7_cli.run_operation'].assert_called_with(self.jboss_config, '/subsystem=datasources/data-source="appDS":add(driver-name="mysql",connection-url="jdbc:mysql://localhost:3306/app",jndi-name="java:jboss/datasources/appDS",user-name="app",password="app_password")', fail_on_error=False)

    def test_create_datasource_format_boolean_value_when_string(self):
        def cli_command_response(jboss_config, cli_command, fail_on_error=False):
            if cli_command == '/subsystem=datasources/data-source="appDS":read-resource-description':
                return {'outcome': 'success',
                        'result': {
                            'attributes': {
                                'use-ccm': {'type': 'BOOLEAN'}
                            }
                        }
                }

        __salt__['jboss7_cli.run_operation'].side_effect = cli_command_response
        datasource_properties = OrderedDict()
        datasource_properties['use-ccm'] = 'true'

        jboss7.create_datasource(self.jboss_config, 'appDS', datasource_properties)

        __salt__['jboss7_cli.run_operation'].assert_called_with(self.jboss_config, '/subsystem=datasources/data-source="appDS":add(use-ccm=true)', fail_on_error=False)

    def test_create_datasource_format_boolean_value_when_boolean(self):
        def cli_command_response(jboss_config, cli_command, fail_on_error=False):
            if cli_command == '/subsystem=datasources/data-source="appDS":read-resource-description':
                return {'outcome': 'success',
                        'result': {
                            'attributes': {
                                'use-ccm': {'type': 'BOOLEAN'}
                            }
                        }
                }

        __salt__['jboss7_cli.run_operation'].side_effect = cli_command_response
        datasource_properties = OrderedDict()
        datasource_properties['use-ccm'] = True

        jboss7.create_datasource(self.jboss_config, 'appDS', datasource_properties)

        __salt__['jboss7_cli.run_operation'].assert_called_with(self.jboss_config, '/subsystem=datasources/data-source="appDS":add(use-ccm=true)', fail_on_error=False)

    def test_create_datasource_format_int_value_when_int(self):
        def cli_command_response(jboss_config, cli_command, fail_on_error=False):
            if cli_command == '/subsystem=datasources/data-source="appDS":read-resource-description':
                return {'outcome': 'success',
                        'result': {
                            'attributes': {
                                'min-pool-size': {'type': 'INT'}
                            }
                        }
                }

        __salt__['jboss7_cli.run_operation'].side_effect = cli_command_response
        datasource_properties = OrderedDict()
        datasource_properties['min-pool-size'] = 15

        jboss7.create_datasource(self.jboss_config, 'appDS', datasource_properties)

        __salt__['jboss7_cli.run_operation'].assert_called_with(self.jboss_config, '/subsystem=datasources/data-source="appDS":add(min-pool-size=15)', fail_on_error=False)

    def test_create_datasource_format_int_value_when_string(self):
        def cli_command_response(jboss_config, cli_command, fail_on_error=False):
            if cli_command == '/subsystem=datasources/data-source="appDS":read-resource-description':
                return {'outcome': 'success',
                        'result': {
                            'attributes': {
                                'min-pool-size': {'type': 'INT'}
                            }
                        }
                }

        __salt__['jboss7_cli.run_operation'].side_effect = cli_command_response
        datasource_properties = OrderedDict()
        datasource_properties['min-pool-size'] = '15'

        jboss7.create_datasource(self.jboss_config, 'appDS', datasource_properties)

        __salt__['jboss7_cli.run_operation'].assert_called_with(self.jboss_config, '/subsystem=datasources/data-source="appDS":add(min-pool-size=15)', fail_on_error=False)

    def test_read_datasource(self):
        def cli_command_response(jboss_config, cli_command):
            if cli_command == '/subsystem=datasources/data-source="appDS":read-resource':
                return {
                    'outcome': 'success',
                    'result': {
                        'driver-name': 'mysql',
                        'connection-url': 'jdbc:mysql://localhost:3306/app',
                        'jndi-name': 'java:jboss/datasources/appDS',
                        'user-name': 'app',
                        'password': 'app_password'
                    }
                }

        __salt__['jboss7_cli.run_operation'].side_effect = cli_command_response

        ds_result = jboss7.read_datasource(self.jboss_config, 'appDS')
        ds_properties = ds_result['result']

        self.assertEqual(ds_properties['driver-name'], 'mysql')
        self.assertEqual(ds_properties['connection-url'], 'jdbc:mysql://localhost:3306/app')
        self.assertEqual(ds_properties['jndi-name'], 'java:jboss/datasources/appDS')
        self.assertEqual(ds_properties['user-name'], 'app')
        self.assertEqual(ds_properties['password'], 'app_password')

    def test_update_datasource(self):
        datasource_properties = {'driver-name': 'mysql',
                                 'connection-url': 'jdbc:mysql://localhost:3306/app',
                                 'jndi-name': 'java:jboss/datasources/appDS',
                                 'user-name': 'newuser',
                                 'password': 'app_password'}

        def cli_command_response(jboss_config, cli_command, fail_on_error=False):
            if cli_command == '/subsystem=datasources/data-source="appDS":read-resource-description':
                return {'outcome': 'success',
                        'result': {
                            'attributes': {
                                'driver-name': {'type': 'STRING'},
                                'connection-url': {'type': 'STRING'},
                                'jndi-name': {'type': 'STRING'},
                                'user-name': {'type': 'STRING'},
                                'password': {'type': 'STRING'}
                            }
                        }
                }

            elif cli_command == '/subsystem=datasources/data-source="appDS":read-resource':
                return {
                    'outcome': 'success',
                    'result': {
                        'driver-name': 'mysql',
                        'connection-url': 'jdbc:mysql://localhost:3306/app',
                        'jndi-name': 'java:jboss/datasources/appDS',
                        'user-name': 'app',
                        'password': 'app_password'
                    }
                }

            elif cli_command == '/subsystem=datasources/data-source="appDS":write-attribute(name="user-name",value="newuser")':
                return {
                    'outcome': 'success',
                    'success': True

                }
        __salt__['jboss7_cli.run_operation'].side_effect = cli_command_response

        jboss7.update_datasource(self.jboss_config, 'appDS', datasource_properties)

        __salt__['jboss7_cli.run_operation'].assert_any_call(self.jboss_config, '/subsystem=datasources/data-source="appDS":write-attribute(name="user-name",value="newuser")', fail_on_error=False)



# -*- coding: utf-8 -*-
'''
tests for host state
'''

# Import python libs
from __future__ import absolute_import
import os
import shutil

# Import Salt Testing libs
from salttesting.helpers import ensure_in_syspath
ensure_in_syspath('../../')

# Import salt libs
import integration
import salt.utils

HFILE = os.path.join(integration.TMP, 'hosts')


class HostTest(integration.ModuleCase, integration.SaltReturnAssertsMixIn):
    '''
    Validate the host state
    '''

    def setUp(self):
        shutil.copyfile(os.path.join(integration.FILES, 'hosts'), HFILE)
        super(HostTest, self).setUp()

    def tearDown(self):
        if os.path.exists(HFILE):
            os.remove(HFILE)
        super(HostTest, self).tearDown()

    def test_present(self):
        '''
        host.present
        '''
        name = 'spam.bacon'
        ip = '10.10.10.10'
        ret = self.run_state('host.present', name=name, ip=ip)
        self.assertSaltTrueReturn(ret)
        with salt.utils.fopen(HFILE) as fp_:
            output = fp_.read()
            self.assertIn('{0}\t\t{1}'.format(ip, name), output)


if __name__ == '__main__':
    from integration import run_tests
    run_tests(HostTest)



# -*- coding: utf-8 -*-
'''
Module for viewing and modifying sysctl parameters
'''
from __future__ import absolute_import

# Import python libs
import os

# Import salt libs
import salt.utils
from salt.exceptions import CommandExecutionError

# Define the module's virtual name
__virtualname__ = 'sysctl'


def __virtual__():
    '''
    Only run on Darwin (OS X) systems
    '''
    return __virtualname__ if __grains__['os'] == 'MacOS' else False


def show(config_file=False):
    '''
    Return a list of sysctl parameters for this minion

    CLI Example:

    .. code-block:: bash

        salt '*' sysctl.show
    '''
    roots = (
        'audit',
        'debug',
        'hw',
        'hw',
        'kern',
        'machdep',
        'net',
        'net',
        'security',
        'user',
        'vfs',
        'vm',
    )
    cmd = 'sysctl -a'
    ret = {}
    out = __salt__['cmd.run'](cmd, output_loglevel='trace', python_shell=False)
    comps = ['']
    for line in out.splitlines():
        # This might need to be converted to a regex, and more, as sysctl output
        # can for some reason contain entries such as:
        #
        # user.tzname_max = 255
        # kern.clockrate: hz = 100, tick = 10000, profhz = 100, stathz = 100
        # kern.clockrate: {hz = 100, tick = 10000, tickadj = 2, profhz = 100,
        #                 stathz = 100}
        #
        # Yes. That's two `kern.clockrate`.
        #
        if any([line.startswith('{0}.'.format(root)) for root in roots]):
            comps = line.split(': ' if ': ' in line else ' = ', 1)
            if len(comps) == 2:
                ret[comps[0]] = comps[1]
            else:
                ret[comps[0]] = ''
        elif comps[0]:
            ret[comps[0]] += '{0}\n'.format(line)
        else:
            continue
    return ret


def get(name):
    '''
    Return a single sysctl parameter for this minion

    name
        The name of the sysctl value to display.

    CLI Example:

    .. code-block:: bash

        salt '*' sysctl.get hw.physmem
    '''
    cmd = 'sysctl -n {0}'.format(name)
    out = __salt__['cmd.run'](cmd, python_shell=False)
    return out


def assign(name, value):
    '''
    Assign a single sysctl parameter for this minion

    name
        The name of the sysctl value to edit.

    value
        The sysctl value to apply.

    CLI Example:

    .. code-block:: bash

        salt '*' sysctl.assign net.inet.icmp.icmplim 50
    '''
    ret = {}
    cmd = 'sysctl -w {0}="{1}"'.format(name, value)
    data = __salt__['cmd.run_all'](cmd, python_shell=False)

    if data['retcode'] != 0:
        raise CommandExecutionError('sysctl failed: {0}'.format(
            data['stderr']))
    new_name, new_value = data['stdout'].split(':', 1)
    ret[new_name] = new_value.split(' -> ')[-1]
    return ret


def persist(name, value, config='/etc/sysctl.conf', apply_change=False):
    '''
    Assign and persist a simple sysctl parameter for this minion

    name
        The name of the sysctl value to edit.

    value
        The sysctl value to apply.

    config
        The location of the sysctl configuration file.

    apply_change
        Default is False; Default behavior only creates or edits
        the sysctl.conf file. If apply is set to True, the changes are
        applied to the system.

    CLI Example:

    .. code-block:: bash

        salt '*' sysctl.persist net.inet.icmp.icmplim 50
        salt '*' sysctl.persist coretemp_load NO config=/etc/sysctl.conf
    '''
    nlines = []
    edited = False
    value = str(value)

    # If the sysctl.conf is not present, add it
    if not os.path.isfile(config):
        try:
            with salt.utils.fopen(config, 'w+') as _fh:
                _fh.write('#\n# Kernel sysctl configuration\n#\n')
        except (IOError, OSError):
            msg = 'Could not write to file: {0}'
            raise CommandExecutionError(msg.format(config))

    with salt.utils.fopen(config, 'r') as ifile:
        for line in ifile:
            if not line.startswith('{0}='.format(name)):
                nlines.append(line)
                continue
            else:
                key, rest = line.split('=', 1)
                if rest.startswith('"'):
                    _, rest_v, rest = rest.split('"', 2)
                elif rest.startswith('\''):
                    _, rest_v, rest = rest.split('\'', 2)
                else:
                    rest_v = rest.split()[0]
                    rest = rest[len(rest_v):]
                if rest_v == value:
                    return 'Already set'
                new_line = '{0}={1}'.format(name, value)
                nlines.append(new_line)
                nlines.append('\n')
                edited = True
    if not edited:
        nlines.append('{0}={1}'.format(name, value))
        nlines.append('\n')
    with salt.utils.fopen(config, 'w+') as ofile:
        ofile.writelines(nlines)
    # If apply_change=True, apply edits to system
    if apply_change is True:
        assign(name, value)
        return 'Updated and applied'
    return 'Updated'



# -*- coding: utf-8 -*-
'''

This module allows you to control the power settings of a windows minion via
powercfg.

.. versionadded:: 2015.8.0

.. code-block:: yaml

    monitor:
        powercfg.set_timeout:
            - value: 30
            - power: dc
'''

# Import Python Libs
from __future__ import absolute_import
import logging

log = logging.getLogger(__name__)

__virtualname__ = "powercfg"


def __virtual__():
    '''
    Only work on Windows
    '''
    if __grains__['os'] == 'Windows':
        return __virtualname__
    return False


def _check_or_set(check_func, set_func, value, power):
    values = check_func()
    if values[power] == value:
        return True
    else:
        set_func(value, power)
        return False


def set_timeout(name, value, power="ac"):
    '''
    Set the sleep timeouts of specific items such as disk, monitor.

    CLI Example:

    .. code-block:: yaml

        monitor:
            powercfg.set_timeout:
                - value: 30
                - power: dc

        disk:
            powercfg.set_timeout:
                - value: 12
                - power: ac

    name
        The setting to change, can be one of the following: monitor, disk, standby, hibernate

    timeout
        The amount of time in minutes before the item will timeout i.e the monitor

    power
        Should we set the value for AC or DC (battery)? Valid options ac,dc.
    '''
    ret = {'name': name,
           'result': True,
           'comment': '',
           'changes': {}}

    comment = []

    if name not in ["monitor", "disk", "standby", "hibernate"]:
        ret["result"] = False
        comment.append("{0} is not a valid setting".format(name))
    elif power not in ["ac", "dc"]:
        ret["result"] = False
        comment.append("{0} is not a power type".format(power))
    else:
        check_func = __salt__["powercfg.get_{0}_timeout".format(name)]
        set_func = __salt__["powercfg.set_{0}_timeout".format(name)]

        values = check_func()
        if values[power] == value:
            comment.append("{0} {1} is already set with the value {2}.".format(name, power, value))
        else:
            ret['changes'] = {name: {power: value}}
            set_func(value, power)

    ret['comment'] = ' '.join(comment)
    return ret
